/*
 * Created at denebola on 22-ago-2004 12:08:33
 *
 */
package element;
import java.util.*;
import java.util.regex.*;


/**
 * <p>Las instancias de la clase SumOfPositionsExpression representan sumas 
 * (y restas) de posiciones de cadena de entrada (StringPosition) y enteros (IntElement)
 * ground, que darn como resultado posiciones de cadena de entrada (StringPosition).
 * Por ejemplo i+j, k-1, etc. (ntese que en k-1, k es StringPosition pero -1 es 
 * IntElement, no hay tal posicin de la cadena)</p>
 * <p>Created at denebola, 22-ago-2004 12:08:33</p>
 * @author carlos
 */
public class SumOfPositionsExpression extends ExpressionElement
{

    /**
     * @param children Sumandos de esta suma de posiciones.
     */
    public SumOfPositionsExpression ( Element[] children )
    {
        super ( children );
    }
   
    /**
     * La expresin suma de posiciones devuelve como cdigo Java la posicin
     * cuyo valor es la suma de los valores de los miembros.
     */
    public String getExpressionCode ( String[] memberNames )
    {
        StringBuffer result = new StringBuffer("");
        result.append(getResultingClass().getName()+".groundFromValue(");
        for ( int i = 0 ; i < memberNames.length ; i++ )
        {
           	result.append("+");
           	result.append("(");
           	result.append(memberNames[i]);
           	result.append(".getValue()");
           	result.append(")");
        }
        result.append(")");
        return result.toString();
    }
    
    public boolean isGround ()
    {
        return false;
    }
    
    /**
     * La clase resultante de una expresin de tipo suma de posiciones es la clase
     * de las posiciones de la cadena de entrada.
     */
    public Class getResultingClass()
    {
        return StringPosition.class;
    }
    
    //calculate children
    protected static List fromStringAux ( String s )
    {
        StringTokenizer st = new StringTokenizer ( s , "+-" , true );
        ArrayList children = new ArrayList();
        boolean positive = true; //signo
        while ( st.hasMoreTokens() )
        {
            String tok = st.nextToken().trim();
            if ( tok.equals("+") ) ;
            if ( tok.equals("-") ) positive = !positive;
            if ( Pattern.compile("[0-9]+").matcher(tok).matches() )
            {
                String intEltTok = tok;
                if ( !positive )
                    intEltTok = "-" + tok;
                children.add( new IntElement(Integer.valueOf(intEltTok).intValue(),true) );
                positive = true;
            }
            if ( Pattern.compile("[i-k]+").matcher(tok).matches() )
            {
                children.add( parsePosition(tok) );
                positive = true;
            }
        }
        return children;
    }
    
    /**
     * Construye una expresin de suma de posiciones a partir de la cadena
     * que la representa.
     * @param s Cadena representando una expresin tipo SumOfPositionsExpression.
     * @return La instancia de SumOfPositionsExpression correspondiente a la
     * cadena.
     */
    public static SumOfPositionsExpression fromString ( String s )
    {
    	List children = fromStringAux(s);
        return new SumOfPositionsExpression( (Element[])children.toArray(new Element[0]) );
    }
    
    protected static StringPosition parsePosition ( String tok )
    {
    	return StringPosition.nonGroundFromString(tok);
    }
    
    class SumOfPositionsExpressionFormatException extends Exception
    {
        public SumOfPositionsExpressionFormatException()
        {
            super();
        }
    }

}
