/*
 * Created at regulus on 10-oct-2004 13:39:44
 * as file SymbolSequence.java on package element
 */
package element;

/**
 * @author carlos
 * 
 * Elemento tipo "SequenceElement" que representa una secuencia de smbolos (cadena).
 * Elementos como stos representan, por ejemplo, alpha y beta en el [A->alpha.Bbeta]
 * que aparece en el algoritmo de Earley.
 * Created at regulus, 10-oct-2004 13:39:44
 */
public class SymbolSequence extends SequenceElement 
{

	/**
	 * Construye un elemento que empareja con una secuencia de smbolos.
	 */
	private SymbolSequence()
	{
		super(Symbol.class);
	}
	
	/**
	 * Lleva una tabla hash que relaciona cadenas (nombres) con instancias de la
	 * clase SymbolSequence.
	 */
	private static java.util.HashMap instances;

	private String name;
	
	/**
	 * Devuelve una instancia de SymbolSequence a partir de una cadena dada.
	 * Esta llamada devuelve instancias diferentes cuando se invoca con cadenas
	 * distintas; pero devuelve la misma instancia siempre que se invoque con
	 * la misma cadena como parmetro.
	 * @param s Cadena que se utilizar para identificar una instancia de SymbolSequence.
	 * @return Instancia identificada por la cadena dada. Si es la primera vez
	 * que se invoca este mtodo con esa cadena, la instancia se crea, de lo contrario
	 * se devuelve la ya existente.
	 */
	public static SymbolSequence fromString ( String s )
	{
		if ( instances == null ) instances = new java.util.HashMap();
		SymbolSequence myInstance = (SymbolSequence) instances.get(s);
		if ( myInstance == null )
		{
			myInstance = new SymbolSequence();
			instances.put(s,myInstance);
			myInstance.name = s;
		}
		return myInstance;
	}
	
	/**
	 * Nos da el nombre con que hemos inicializado esta instancia de SymbolSequence.
	 */
	public String toString()
	{
		return (String) name;
	}
	
	/**
	 * Comparacin de igualdad de secuencias de smbolos.
	 */
	public boolean equals ( Object another )
	{
		if ( !(another instanceof SymbolSequence) ) return false;
		return this.name.equals(((SymbolSequence)another).name);
	}
	
	
}
