package element.fs;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;

import java.util.List;

import element.Element;
import element.ExpressionElement;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Represents an expression of the form Tree-Variables(node) which gets
 * a feature structure with all the variables associated to a given tree.
 * @author Carlos
 *
 */
public class TreeVariablesExpression extends ExpressionElement
{

		public TreeVariablesExpression(Element argument)
		{
			super( new Element[] { argument } );
		}
		
		public boolean isGround()
		{
			return false;
		}
		
		public Class getResultingClass()
		{
			return PlainFeatureStructure.class;
		}
		
		public String getExpressionCode(String[] memberNames) 
		{			
			StringBuffer sb = new StringBuffer("");
			sb.append("((grammar.tag.TreeAdjoiningGrammarWithFeatureStructures)");
			sb.append(memberNames[0]);
			sb.append(".getGrammar()).getTreeVariablesStructure(");
			sb.append(memberNames[0]);
			sb.append(")");
			return new String(sb);
		}
		
		public static TreeVariablesExpression fromString ( String sparm ) throws ElementParseException
		{
			try
			{
				FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
				List expressionArgs = fep.getArguments();
				if ( expressionArgs.size() != 1 )
					throw new ElementParseException("Tree Variables expression taking " + expressionArgs.size() + " arguments when 1 are expected");
				Element argument = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
				return new TreeVariablesExpression ( argument );
				
			}
			catch ( FunctionExpressionParseException fepe )
			{
				throw new ElementParseException(fepe);
			}
		}

		
}
