package element.fs;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import element.tag.TAGTree;

/**
 * This class stores a word, its corresponding POS tag and features.
 */
public class WordInformation 
{
	
	private String word;
	private String lemma;
	private String tag;
	private PlainFeatureStructure features;
	
	public WordInformation(String word, String tag, String lemma, PlainFeatureStructure features) 
	{
		this.word = word;
		this.tag = tag;
		this.lemma = lemma;
		this.features = features;
	}
	
	
	private List trees = new ArrayList();
	private List families = new ArrayList();
	
	
	public String xtagTagConversion ( String tag )
	{
		if ( "Det".equals(tag) ) return "D";
		if ( "P1".equals(tag) ) return "P";
		if ( "P2".equals(tag) ) return "P";
		if ( "Punct1".equals(tag) ) return "Punct";
		if ( "Punct2".equals(tag) ) return "Punct";
		if ( "N1".equals(tag) ) return "N";
		if ( "D1".equals(tag) ) return "D";
		if ( "D2".equals(tag) ) return "D";
		if ( "Conj1".equals(tag) ) return "Conj";
		if ( "Conj2".equals(tag) ) return "Conj";
		if ( "Adv".equals(tag) ) return "Ad";
		if ( "PropN".equals(tag) ) return "N";
		if ( "Pron".equals(tag) ) return "N";
		if ( "Prep".equals(tag) ) return "P";
		if ( "Part".equals(tag) ) return "PL";
		return tag;
	}
	
	public void getXtagTreeFilteringInformation()
	{
		
		File f = new File("C:\\nlpresources\\XTAGold99\\english\\syntax\\syntax-coded.db.flat");
		BufferedReader br = null;
		try
		{
			br = new BufferedReader ( new InputStreamReader ( new FileInputStream(f) ) );
			String linea;
			while ( (linea = br.readLine())!=null )
			{
				
				int indexStart = linea.indexOf("<<INDEX>>") + "<<INDEX>>".length();
				int indexEnd = linea.indexOf("<<",indexStart);
				if ( indexStart >= 0 && indexEnd < 0 ) indexEnd = linea.length();
				
				int entryStart = linea.indexOf("<<ENTRY>>") + "<<INDEX>>".length();
				int entryEnd = linea.indexOf("<<",entryStart);
				if ( entryStart >= 0 && entryEnd < 0 ) entryEnd = linea.length();
				
				int posStart = linea.indexOf("<<POS>>") + "<<POS>>".length();
				int posEnd = linea.indexOf("<<",posStart);
				if ( posStart >= 0 && posEnd < 0 ) posEnd = linea.length();
				
				int familyStart = linea.indexOf("<<FAMILY>>") + "<<FAMILY>>".length();
				int familyEnd = linea.indexOf("<<",familyStart);
				if ( familyStart >= 0 && familyEnd < 0 ) familyEnd = linea.length();
				
				int treesStart = linea.indexOf("<<TREES>>") + "<<TREES>>".length();
				int treesEnd = linea.indexOf("<<",treesStart);
				if ( treesStart >= 0 && treesEnd < 0 ) treesEnd = linea.length();
				
				int featuresStart = linea.indexOf("<<FEATURES>>") + "<<FEATURES>>".length();
				int featuresEnd = linea.indexOf("<<",featuresStart);
				if ( featuresStart >= 0 && featuresEnd < 0 ) featuresEnd = linea.length();
				
				String index = null;
				if ( indexStart >= "<<INDEX>>".length() )
					index = linea.substring(indexStart,indexEnd);
				String entry = null;
				if ( entryStart >= "<<ENTRY>>".length() )
					entry = linea.substring(entryStart,entryEnd);
				String pos = null;
				if ( posStart >= "<<POS>>".length() )
					pos = linea.substring(posStart,posEnd);
				String family = null;
				if ( familyStart >= "<<FAMILY>>".length() )
					family = linea.substring(familyStart,familyEnd);
				String trees = null;
				if ( treesStart >= "<<TREES>>".length() )
					trees = linea.substring(treesStart,treesEnd);
				String features = null;
				if ( featuresStart >= "<<FEATURES>>".length() )
					features = linea.substring(featuresStart,featuresEnd);
						
				
				if ( index.equals(lemma) && pos.equals(xtagTagConversion(tag)) )
				{
					
					System.out.println("Word: " + word);
					System.out.println("Index: " + index);
					System.out.println("Entry: " + entry);
					System.out.println("Pos: " + pos);
					System.out.println("Family: " + family);
					System.out.println("Trees: " + trees);
					System.out.println("Features: ((" + features + "))");
					System.out.println("Base Feats: " + this.features);
					
					if ( family != null )
					{
						StringTokenizer stFam = new StringTokenizer(family);
						while ( stFam.hasMoreTokens() )
						{
							families.add(stFam.nextToken().trim());
						}
					}
					
					if ( trees != null )
					{
						StringTokenizer stTree = new StringTokenizer(trees);
						while ( stTree.hasMoreTokens() )
						{
							String treeName = stTree.nextToken().trim();
							if ( ! Character.isLetterOrDigit(treeName.charAt(0)) )
								treeName = treeName.substring(1);
							this.trees.add(treeName.toUpperCase());
							System.out.println(treeName.toUpperCase());
						}
					}
					
				}
				
			}
		}
		catch ( Throwable t )
		{
			t.printStackTrace();
		}
		
	}
	
	//xtag tree filter
	public boolean selectsTree ( TAGTree tree )
	{
		if ( trees == null && families == null ) getXtagTreeFilteringInformation();
		if ( trees.contains(tree.getName()) ) return true;
		else if ( families.contains(tree.getFamily()) ) return true;
		else if ( trees == null && families == null ) return true;
		else return false;
	}
	

	public PlainFeatureStructure getFeatures() 
	{
		return features;
	}

	public void setFeatures(PlainFeatureStructure features) 
	{
		this.features = features;
	}
	
	public String getLemma()
	{
		return lemma;
	}
	
	public void setLemma(String lemma)
	{
		this.lemma = lemma;
	}

	public String getTag() 
	{
		return tag;
	}

	public void setTag(String tag) 
	{
		this.tag = tag;
	}

	public String getWord() 
	{
		return word;
	}

	public void setWord(String word) 
	{
		this.word = word;
	}
	
	

}
