package element.predicate;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.SumOfIntegersExpression;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Represents a boolean expression of the form Null(something) telling us if
 * the parameter is null.
 */
public class NullExpression extends ExpressionElement 
{

	public NullExpression(Element argument1 )
	{
		super( new Element[] { argument1 } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return element.BooleanElement.class;
	}
	
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("");
        result.append("BooleanElement.groundFromValue(");
        result.append(memberNames[0]);
        result.append("==null");
        result.append(")");
        return result.toString();
	}
	
	public static NullExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() != 1 )
			{
				throw new ElementParseException("Null expression taking " + expressionArgs.size() + " arguments when 1 is expected");
			}
			
			Element argument1 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			
			return new NullExpression ( argument1 );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
