package grammar.tag.util;

import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class GrammarGenerator 
{

	public static void main ( String[] args ) throws Throwable
	{
	
		
		int NUM_TREES = 512;
		String fileName="mytag"+NUM_TREES+".xml";
		
		DocumentBuilder db = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		Document d = db.newDocument();
		
		//root element
		Element rootElement = d.createElement("tag");
		rootElement.setAttribute("axiom","S");
		rootElement.setAttribute("familyfile",fileName);
		rootElement.setAttribute("lemmafile",fileName);
		rootElement.setAttribute("lexfile",fileName);
		d.appendChild(rootElement);
		
		//family element
		Element familyElement = d.createElement("family");
		familyElement.setAttribute("name","trees");
		rootElement.appendChild(familyElement);
		
		//initial tree element
		Element initialTreeElement = d.createElement("tree");
		initialTreeElement.setAttribute("name","initial");
		Element initialTreeRoot = d.createElement("node");
		initialTreeRoot.setAttribute("cat","S");
		initialTreeRoot.setAttribute("id","S_r");
		initialTreeRoot.setAttribute("type","none");
		Element initialTreeAdj = d.createElement("node");
		initialTreeAdj.setAttribute("cat","A");
		initialTreeAdj.setAttribute("id","A");
		initialTreeAdj.setAttribute("type","adj");
		Element initialTreeTerminal = d.createElement("node");
		initialTreeTerminal.setAttribute("cat","a0");
		initialTreeTerminal.setAttribute("id","terminal");
		initialTreeTerminal.setAttribute("type","anchor");
		
		//link initial tree
		initialTreeRoot.appendChild(initialTreeAdj);
		initialTreeAdj.appendChild(initialTreeTerminal);
		initialTreeElement.appendChild(initialTreeRoot);
		familyElement.appendChild(initialTreeElement);
		
		for ( int i = 1 ; i <= NUM_TREES ; i++ )
		{
			
			//auxiliary tree element
			Element auxTreeElement = d.createElement("tree");
			auxTreeElement.setAttribute("name","aux"+i);
			Element auxTreeRoot = d.createElement("node");
			auxTreeRoot.setAttribute("cat","A");
			auxTreeRoot.setAttribute("id","A_r");
			auxTreeRoot.setAttribute("type","none");
			Element auxTreeAdj = d.createElement("node");
			auxTreeAdj.setAttribute("cat","A");
			auxTreeAdj.setAttribute("id","A_a");
			auxTreeAdj.setAttribute("type","adj");
			Element auxTreeFoot = d.createElement("node");
			auxTreeFoot.setAttribute("cat","A");
			auxTreeFoot.setAttribute("id","A_f");
			auxTreeFoot.setAttribute("type","foot");
			Element auxTreeTerminal = d.createElement("node");
			auxTreeTerminal.setAttribute("cat","a"+i);
			auxTreeTerminal.setAttribute("id","terminal");
			auxTreeTerminal.setAttribute("type","anchor");
			
			//link auxiliary tree
			auxTreeRoot.appendChild(auxTreeAdj);
			auxTreeAdj.appendChild(auxTreeFoot);
			auxTreeAdj.appendChild(auxTreeTerminal);
			auxTreeElement.appendChild(auxTreeRoot);
			familyElement.appendChild(auxTreeElement);
			
		}
		
		System.out.println("Going to write XML output...");
		
		Transformer t = TransformerFactory.newInstance().newTransformer();
		t.transform(new DOMSource(d),new StreamResult(new File("C:\\Proyecto\\Version9\\Grammars\\"+fileName)));
		
		System.out.println("Done.");

	}
	
}
