/*
 * Created at regulus on 06-mar-2005 18:19:57
 * as file ParserInterface.java on package test
 */
package test;

import element.Element;
import element.StringPosition;
import element.Symbol;
import grammar.Grammar;
import grammar.IncorrectGrammarFormatException;
import grammar.GrammarLoader;
import agenda.Agenda;
import agenda.AgendaLoader;
import engine.DeductiveEngine;
import engine.DeductiveEngineLoader;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;

import schema.item.ItemHandler;
import schema.item.Items;
import schema.step.DeductiveStep;
import schema.step.StepFactory;
import schema.step.StepHandler;
import element.SymbolTable;

/**
 * @author carlos
 *
 * Created at regulus, 06-mar-2005 18:19:57
 */
public class ParserInterface extends JFrame
{

	private JTextField gramTextField = new JTextField(40);
	private JButton gramButton = new JButton("Examinar...");
	private JButton gramButton2 = new JButton("Cargar");
	private JTextField sentenceTextField = new JTextField(40);
	private JButton parseButton = new JButton("Analizar");
	
	private JCheckBox goalCheckBox = new JCheckBox("Mostrar slo items meta");
	
	private Grammar grammar;
	
	private List deductiveSteps;
	private List goalConditions;
	
	private OutputJTextArea logArea;
	private OutputJTextArea resultArea;
	
	private long timeRef;
	
	private Agenda agenda = AgendaLoader.getAgenda();
	
	private DeductiveEngine engine;
		
		
	private void startTimeCount()
	{
		timeRef = System.currentTimeMillis();
	}
	private long getTimeCount()
	{
		return System.currentTimeMillis()-timeRef;
	}
	
	private void loadGrammarFromFile ( String fileName )
	{
		logArea.println("Preparando algoritmo para gramtica...");
		//grammar = new ContextFreeGrammar(new File(fileName));
		
		try
		{
			grammar = GrammarLoader.getGrammar ( new File(fileName) );
		}
		catch ( Throwable t )
		{
			System.err.println("Error reading grammar from file " + fileName + ": ");
			t.printStackTrace();
		}
		
		//instanciar los pasos deductivos y StepHandler
	
		startTimeCount();
		
		deductiveSteps = StepFactory.instanceSteps(grammar);
		goalConditions = StepFactory.instanceGoalConditions(grammar);
		new ItemHandler();
		new StepHandler();
		int sz = deductiveSteps.size();
		for ( int i = 0 ; i < sz ; i++ )
		{
			StepHandler.instance().addStep((DeductiveStep)deductiveSteps.get(i));
		}
		engine = DeductiveEngineLoader.getEngine( agenda , deductiveSteps , goalConditions , ItemHandler.instance() , StepHandler.instance() );
		
		long time = getTimeCount();
		logArea.println(deductiveSteps.size() + " pasos deductivos instanciados en " + time + " ms");
	
		
		//ejecutar la mquina deductiva para hacer todo aquello que se pueda hacer sin la frase
		startTimeCount();
		engine.preprocess();
		time = getTimeCount();
		logArea.println(ItemHandler.instance().getAll().size() + " items computados en " + time + " ms"); 
		
		ItemHandler.fastSave();
		
		logArea.println("Gramtica preparada.");
		
		SwingUtilities.invokeLater ( new Runnable() 
		{
			public void run()
			{
				parseButton.setEnabled(true);
			}
		}
		);
		
	}
	
	private void processSentence ( String sentence )
	{	
		StringTokenizer st = new StringTokenizer( sentence );
		//leer terminales
		List tokens = new ArrayList(); //of items
		int sentencelen = 0;
		//apply k-function to generate items from sentence!!
		while ( st.hasMoreTokens() )
		{
			String tokText = st.nextToken().trim();
			Symbol tokSym = SymbolTable.instance().symbolFor(tokText.replace(':','%').trim());
			Element[] item = new Element[3];
			item[0] = tokSym;
			item[1] = StringPosition.groundFromValue(sentencelen);
			sentencelen++;
			item[2] = StringPosition.groundFromValue(sentencelen);
			tokens.add(item);
		}
		StringPosition.setMaxValue(sentencelen);
		//new ItemHandler();
		startTimeCount();
		ItemHandler.fastRestore();
		long time = getTimeCount();
		logArea.println("ItemHandler restaurada en " + time + " ms");
		for ( int i = 0 ; i < tokens.size() ; i++ )
		{
			ItemHandler.instance().addItem((Element[])tokens.get(i));
			agenda.add ( (Element[]) tokens.get(i) );
		}
		
		resultArea.clear();
		engine.setItemHandler(ItemHandler.instance()); //this is due to fastRestore() calling new ItemHandler() and changing the registered instance
		startTimeCount();
		engine.process();
		time = getTimeCount();
		logArea.printlnNow(ItemHandler.instance().getAll().size() + " items computados en " + time + " ms"); 
		
		//sacar items por el rea de items
		List lItems = ItemHandler.instance().getAll();
		
		if ( !goalCheckBox.isSelected() )
		{
			resultArea.printlnNow("Items obtenidos:");
			
			StringBuffer accum = new StringBuffer("");
			
			for ( int i = 0 ; i < lItems.size() ; i++ )
			{
				Object[] item = (Object[]) lItems.get(i);
			
				accum.append("\n"+Items.toString(item));
				//resultArea.println(Items.toString(item));
				if ( i % 1000 == 999 )
				{
					resultArea.println(accum.toString());
					accum = new StringBuffer("");
				}
			}
			resultArea.println(accum.toString());
			resultArea.println("Fin de lista de items.");
		}
		else
		{
			resultArea.printlnNow("Items finales:");
			List goalConditions = StepFactory.instanceGoalConditions(grammar);
			boolean found = false;
			for ( int i = 0 ; i < lItems.size() ; i++ )
			{
				Object[] item = (Object[]) lItems.get(i);
				if ( goalConditions == null || goalConditions.size() == 0 )
					resultArea.println(Items.toString(item));
				else
				{
					for ( int j = 0 ; j < goalConditions.size() ; j++ )
					{
						List result = ((DeductiveStep)goalConditions.get(j)).applyTo(item);
						if ( result != null && result.size() > 0 )
						{
							found = true;
							resultArea.println(Items.toString(item));
							break; //pasamos al siguiente item, no queremos imprimir ste varias veces aunque cumpla varias condiciones
						}
					}
				}
			}
			if ( !found )
				resultArea.printlnNow("No hay items finales.");
		}
		
	}
		
	
	public ParserInterface()
	{
		
		this.getContentPane().setLayout(new BorderLayout());
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setTitle("Interfaz para el analizador generado");
		
		JPanel panelCampos = new JPanel( new GridLayout(3,1) );
		JPanel subPanelCampos1 = new JPanel(); panelCampos.add(subPanelCampos1);
		JPanel subPanelCampos2 = new JPanel(); panelCampos.add(subPanelCampos2);
		JPanel subPanelCampos3 = new JPanel(); panelCampos.add(subPanelCampos3);
		
		//subPanelCampos1.setLayout(new BorderLayout());
		subPanelCampos1.add(new JLabel("Gramtica:"));
		//JPanel fieldPanel = new JPanel();
		//fieldPanel.add(gramTextField);
		subPanelCampos1.add(gramTextField);
		subPanelCampos1.add(gramButton);
		subPanelCampos1.add(gramButton2);
		
		subPanelCampos2.add(new JLabel("Frase:"));
		subPanelCampos2.add(sentenceTextField);
		subPanelCampos2.add(parseButton);
		
		subPanelCampos3.add(goalCheckBox);
		
		this.getContentPane().add(panelCampos,BorderLayout.NORTH);
		
		JPanel panelAreas = new JPanel ( new GridLayout(1,2) );
		logArea = new OutputJTextArea();
		resultArea = new OutputJTextArea();
		
		final JPopupMenu taMenu = new JPopupMenu();
		JMenuItem item = new JMenuItem("Select All");
		ActionListener al = new ActionListener()
		{
			public void actionPerformed ( ActionEvent evt )
			{
				resultArea.selectAll();
			}
		};
		item.addActionListener(al);
		taMenu.add(item);
		
		resultArea.addMouseListener(new MouseAdapter(){
			
		    public void mousePressed(MouseEvent e) {
		       evaluatePopup(e);
		    }
		 
		    public void mouseReleased(MouseEvent e) {
		       evaluatePopup(e);
		    }
		 
		    private void evaluatePopup(MouseEvent e) {
		       if (e.isPopupTrigger()) {
		    	   taMenu.show(e.getComponent(), e.getX(), e.getY());
		       }
		    }
		});
		
		JScrollPane logScroll = new JScrollPane(logArea);
		JScrollPane resultScroll = new JScrollPane(resultArea);
		panelAreas.add(logScroll);
		panelAreas.add(resultScroll);
		
		this.getContentPane().add(panelAreas,BorderLayout.CENTER);
		
		gramButton.addActionListener( new ActionListener() 
				{
					public void actionPerformed ( ActionEvent evt )
					{
						final JFileChooser jfc = new JFileChooser();
						jfc.setCurrentDirectory(new File("."));
						int retVal = jfc.showOpenDialog(ParserInterface.this);
						if ( retVal == JFileChooser.APPROVE_OPTION )
						{
							gramTextField.setText( jfc.getSelectedFile().getAbsolutePath() );
						}
					}
				}
				);
		
		gramButton2.addActionListener ( new ActionListener() 
				{
					public void actionPerformed ( ActionEvent evt )
					{
						final String fName = gramTextField.getText();
						if ( (new File(fName)).canRead() )
						{
							Thread thr = ( new Thread()
							{
								public void run()
								{
									synchronized ( agenda )
									{
										ParserInterface.this.loadGrammarFromFile ( fName );												
									}
								}
							}
							);
							thr.start();
						}
						else
						{
							JOptionPane.showMessageDialog(ParserInterface.this,"No se puede leer la gramtica del fichero " + fName,"Error",JOptionPane.ERROR_MESSAGE);
						}
					}
			});
		
		parseButton.addActionListener ( new ActionListener()
				{
					public void actionPerformed ( ActionEvent evt )
					{
						if ( grammar == null )
						{
							JOptionPane.showMessageDialog(ParserInterface.this,"No hay gramtica cargada","Error",JOptionPane.ERROR_MESSAGE);
						}
						else
						{
							Thread thr = new Thread() 
							{
								public void run()
								{
									synchronized ( agenda )
									{
										processSentence ( sentenceTextField.getText() );
									}
								}
							};
							thr.start();
						}
					}
				}
			);
		
		parseButton.setEnabled(false);
		
		this.setSize(750,400);
		this.setVisible(true);
		
	}
		
	public static void main(String[] args) 
	{
		new ParserInterface();
	}
}
