/*
 * Created at regulus on 09-oct-2004 13:35:32
 * as file SequenceElement.java on package element
 */
package element;

import java.util.HashMap;
import java.util.Map;

/**
 * @author carlos
 *
 * Esta clase representa una secuencia de cero o ms elementos de alguna otra clase.
 * El nmero de elementos es desconocido mientras la secuencia no est instanciada.
 * Esto se utiliza para representar items con cadenas como los que aparecen en el algoritmo 
 * de Earley, vase [ A -> alpha B . beta ]: alpha y beta seran SequenceElements
 * de Symbols.
 * Created at regulus, 09-oct-2004 13:35:32
 */
public class SequenceElement extends CompositeElement
{

	/**
	 * Clase asociada a los hijos del elemento ZeroOrMore.
	 */
	private Class childrenClass;
	
	/**
	 * Tabla hash que almacena instancias compartidas de SequenceElement.
	 */
	private static Map sharedInstances = new HashMap();
	
	/**
	 * Devuelve una instancia de SequenceElement representando una secuencia de
	 * elementos de la clase dada. Se garantiza que llamadas sucesivas a este mtodo
	 * con la misma clase como parmetro devolvern la misma instancia.
	 * @param cl Subclase de SimpleElement.
	 * @return SequenceElement que representa una secuencia de elementos de la clase
	 * cl.
	 */
	public static SequenceElement fromClass ( Class cl )
	{
		SequenceElement zom = (SequenceElement) sharedInstances.get(cl);
		if ( zom != null )
		{
			return zom;
		}
		else
		{
			zom = new SequenceElement(cl);
			sharedInstances.put(cl,zom);
			return zom;
		}
	}
	
	/**
	 * Crea un elemento que representa cero o ms ocurrencias de elementos
	 * de la clase dada.
	 * @param cl Clase de los elementos que contiene este Sequence.
	 */
	public SequenceElement ( Class cl )
	{
		this.childrenClass = cl;
	}
	
	/**
	 * Devuelve false.
	 */
	public boolean isGround() 
	{
		return false; //siempre no-ground
	}
	
	/**
	 * Nos devuelve la clase que deben tener los elementos que emparejen con este
	 * sequence. Por ejemplo, esta clase ser Symbol para Sequences que
	 * representen cadenas. Lo que es lo mismo, en la frase "Este Sequence
	 * representa cero o ms objetos de la clase X", esto nos devuelve X.
	 * @return Clase asociada a los elementos de este ZeroOrMore.
	 */
	public Class getAssociatedClass()
	{
		return childrenClass;
	}

	/**
	 * Como estos elementos se utilizarn siempre como no-ground, no tienen hijos.
	 * (devuelve 0)
	 */
	public int numChildren() 
	{
		return 0;
	}

	/**
	 * Como estos elementos se utilizarn siempre como no-ground, no tienen hijos.
	 * (devuelve null)
	 */
	public Element getChild(int i) 
	{
		return null;
	}
	
	/**
	 * En el caso de un elemento tipo Sequence,
	 * el elemento hoja es l mismo, ya que en la representacin del esquema
	 * de anlisis
	 * no incluimos nada dentro de l.
	 */
	public java.util.List getLeafElements()
	{
		java.util.ArrayList al = new java.util.ArrayList();
		al.add(this);
		return al;
	}
	
	/**
	 * Hace que los visitadores de generacin de cdigo llamen al mtodo
	 * especfico para los elementos tipo Sequence.
	 */
	public void accept ( ElementVisitor v )
	{
		v.visitSequenceElement(this);
	}
	

}
