package element.fs;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Esta clase representa una expresin booleana de tipo Unify(fs1;fs2;...;fsN)
 * que nos devuelve el resultado de unificar los rasgos de fs2 en fs1, luego
 * los de fs3 en el resultado, etc. etc.
 * @author Carlos
 */
public class StaticUnifyExpression extends ExpressionElement 
{

	public StaticUnifyExpression(Element[] arguments)
	{
		super( arguments );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return PlainFeatureStructure.class;
	}
	
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("element.fs.PlainFeatureStructure.unify(");
        result.append(memberNames[0]);
        result.append(",");
        result.append(memberNames[1]);
        result.append(")");
        for ( int i = 2 ; i < memberNames.length ; i++ )
        {
        	result.insert(0,"element.fs.PlainFeatureStructure.unify(");
        	result.append(",");
        	result.append(memberNames[i]);
        	result.append(")");
        }
        return result.toString();
	}
	
	public static StaticUnifyExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() < 2 )
			{
				throw new ElementParseException("Static Unify expression taking " + expressionArgs.size() + " arguments when 2 or more are expected");
			}
			
			Element[] arguments = new Element[expressionArgs.size()];
			for ( int i = 0 ; i < arguments.length ; i++ )
			{
				arguments[i] = ElementParser.getCurrentInstance().parse((String)expressionArgs.get(i));
			}
			
			return new StaticUnifyExpression ( arguments );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
