/*
 * Created at regulus on 10-oct-2004 13:39:44
 * as file SymbolSequence.java on package element
 */
package element.tag;

import element.SequenceElement;

/**
 * SequenceElement representing a sequence of TAG tree nodes.
 */
public class TAGTreeNodeSequence extends SequenceElement 
{

	/**
	 * Construye un elemento que empareja con una secuencia de nodos TAG.
	 */
	private TAGTreeNodeSequence()
	{
		super(TAGTreeNode.class);
	}
	
	/**
	 * Lleva una tabla hash que relaciona cadenas (nombres) con instancias de la
	 * clase TAGTreeNodeSequence.
	 */
	private static java.util.HashMap instances;

	private String name;
	
	/**
	 * Devuelve una instancia de TAGTreeNodeSequence a partir de una cadena dada.
	 * Esta llamada devuelve instancias diferentes cuando se invoca con cadenas
	 * distintas; pero devuelve la misma instancia siempre que se invoque con
	 * la misma cadena como parmetro.
	 * @param s Cadena que se utilizar para identificar una instancia de TAGTreeNodeSequence.
	 * @return Instancia identificada por la cadena dada. Si es la primera vez
	 * que se invoca este mtodo con esa cadena, la instancia se crea, de lo contrario
	 * se devuelve la ya existente.
	 */
	public static TAGTreeNodeSequence fromString ( String s )
	{
		if ( instances == null ) instances = new java.util.HashMap();
		TAGTreeNodeSequence myInstance = (TAGTreeNodeSequence) instances.get(s);
		if ( myInstance == null )
		{
			myInstance = new TAGTreeNodeSequence();
			instances.put(s,myInstance);
			myInstance.name = s;
		}
		return myInstance;
	}
	
	/**
	 * Nos da el nombre con que hemos inicializado esta instancia de TAGTreeNodeSequence.
	 */
	public String toString()
	{
		return (String) name;
	}
	
	/**
	 * Comparacin de igualdad de secuencias de TAGTreeNodes.
	 */
	public boolean equals ( Object another )
	{
		if ( !(another instanceof TAGTreeNodeSequence) ) return false;
		return this.name.equals(((TAGTreeNodeSequence)another).name);
	}
	
	
}
