/*
 * Created at regulus on 05-feb-2006 11:44:00
 * as file LCRCContextFreeGrammar.java on package grammar.lcrc
 */
package grammar.lcrc;

import java.io.File;
import java.io.IOException;

import element.BooleanElement;
import element.Element;
import element.Symbol;
import element.SymbolTable;
import grammar.CFGUtils;
import grammar.ContextFreeGrammar;
import grammar.Grammar;
import grammar.IncorrectGrammarFormatException;

/**
 * @author carlos
 *
 * Created at regulus, 05-feb-2006 11:44:00
 */
public class LCRCContextFreeGrammar implements Grammar
{

	private ContextFreeGrammar cfg;
	
	
	//Right Corner Table (like Left Corner Table @ CFGUtils, but don't want
	//to clutter CFGUtils with this testing code)
	private boolean[][] rightCornerTable;
	private boolean rightCornerTableInitted = false;
	
	public void initRightCornerTable()
	{
		
		System.out.println("Computing Right-Corner Relationships...");
		long iniTime = System.currentTimeMillis();
		
		int nSymbols = SymbolTable.instance().getLastId()+1;
		rightCornerTable = new boolean[nSymbols][nSymbols];
		
		//reflexivity
		for ( int i = 0 ; i < nSymbols ; i++ )
			rightCornerTable[i][i] = true;
		
		//transitivity
		boolean changed = true;
		while ( changed )
		{
			changed = false;
			for ( int i = 0 ; i < cfg.size() ; i++ )
			{
				Element[] rule = cfg.getEntity(i);
	
				for ( int aind = 0 ; aind < nSymbols ; aind++ )
				{
					if ( rightCornerTable[aind][((Symbol)rule[0]).getID()] )
					{
						if ( rule.length > 1 ) //non-epsilon rule
						{
							if ( rightCornerTable[aind][((Symbol)rule[rule.length-1]).getID()] == false )
								changed = true;
							rightCornerTable[aind][((Symbol)rule[rule.length-1]).getID()] = true;					
						}
					}
				}
			}
		}
		
		long elTime = System.currentTimeMillis() - iniTime;
		System.out.println("RC Relationships computed in " + elTime + "ms.");
		
	}
	
	public int countRightCornerSetFor ( Symbol s1 )
	{
		if ( !s1.isGround() )
		{
			System.err.println("[ERROR] Left-Corner query performed on nonground symbol");
		}
		if ( !rightCornerTableInitted )
		{
			initRightCornerTable();
			rightCornerTableInitted = true;
		}
		boolean [] bs = rightCornerTable[s1.getID()];
		int counter = 0;
		for ( int i = 0 ; i < bs.length ; i++ )
		{
			if ( bs[i] )
				counter++;
		}
		return counter;
	}
	
	public BooleanElement rightCorner ( Symbol s1 , Symbol s2 )
	{
		if ( !s1.isGround() || !s2.isGround() )
		{
			System.err.println("[ERROR] Left-Corner query performed on nonground symbol(s)");
		}
		if ( !rightCornerTableInitted )
		{
			initRightCornerTable();
			rightCornerTableInitted = true;
		}
		boolean b = rightCornerTable[s1.getID()][s2.getID()];
		return BooleanElement.groundFromValue(b);
	}
	
	
	public static LCRCContextFreeGrammar instance;
	
	public static void registerInstance (LCRCContextFreeGrammar inst)
	{
		instance = inst;
	}
	
	public void initFromFile(File f) throws IOException, IncorrectGrammarFormatException 
	{
		cfg = new ContextFreeGrammar();
		cfg.initFromFile(f);	
		cache = new Object[cfg.size()];
		registerInstance(this);
	}

	public int ALWAYS_LEFT = 0;
	public int ALWAYS_RIGHT = 1;
	public int BEST_CORNER = 2;
	
	private int cornerCriterion = 2;
	
	public boolean setLeftCornerCriterion()
	{
		cornerCriterion = 0;
		return true;
	}
	
	public boolean setRightCornerCriterion()
	{
		cornerCriterion = 1;
		return true;
	}
	
	public boolean setBestCornerCriterion()
	{
		cornerCriterion = 2;
		return true;
	}
	
	//decides if a non-augmented rule is left or right
	public boolean isLeft ( Element[] e )
	{	
		if ( cornerCriterion == ALWAYS_LEFT ) return true;
		else if ( cornerCriterion == ALWAYS_RIGHT ) return false;
		else
		{
			int left = CFGUtils.countLeftCornerSetFor ( (Symbol)e[0] );
			int right = this.countRightCornerSetFor ( (Symbol)e[0] );
			return ( left >= right );
		}
	}
	
	private Object[] cache;
	
	
	public Element[] getEntity(int i) 
	{
		if ( cache[i] != null ) return (Element[])cache[i];
		Element[] rule = cfg.getEntity(i);
		Element[] newRule = new Element[rule.length + 1];
		if ( isLeft(rule) )
			newRule[0] = SymbolTable.instance().symbolFor("Left");
		else
			newRule[0] = SymbolTable.instance().symbolFor("Right");
		for ( int j = 0 ; j < rule.length ; j++ )
		{
			if ( isLeft(rule) )
				newRule[j+1] = rule[j];
			else
			{
				if ( j == 0 )
					newRule[j+1] = rule[j];
				else
					newRule[j+1] = rule[rule.length-j];
			}
		}
		cache[i] = newRule;
		return newRule;
	}
	
	public int size() 
	{
		return cfg.size();
	}


}
