package grammar.tag;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import util.DOMNodeListWrapper;
import util.DOMNodeWrapper;

import element.Element;
import element.Symbol;
import element.SymbolTable;
import element.tag.TAGTree;
import element.tag.TAGTreeNode;
import grammar.*;

public class TreeAdjoiningGrammar implements Grammar
{

	private Symbol axiom;
	
	private ArrayList trees = new ArrayList();
	
	private ArrayList rules = new ArrayList(); //CFG style rules. These are the entities of TAG's.
	
	
	public void initFeatures ( TAGTreeNode tagNode , DOMNodeWrapper domNode )
	{
		//default implementation is empty, this is a template method to implement 
		//by subclasses which support feature structures
		;
	}
	
	//test, driver method
	public static void main ( String[] args ) throws Throwable
	{
		TreeAdjoiningGrammar tag = new TreeAdjoiningGrammar();
		tag.initFromFile( new File("C:\\proyecto\\version8\\grammars\\tag\\trees.xml") );
		
		int iniTrees = 0;
		int nNodes = 0;
		for ( int i = 0 ; i < tag.trees.size() ; i++ )
		{
			if ( ((TAGTree)tag.trees.get(i)).isInitial() )
				iniTrees++;
			nNodes += ((TAGTree)tag.trees.get(i)).numNodes();
		}
		
		System.out.println("Trees: " + tag.trees.size());
		System.out.println("Initial trees: " + iniTrees);
		System.out.println("Auxiliary trees: " + (tag.trees.size()-iniTrees));
		System.out.println("Tree nodes: " + nNodes);
		System.out.println("CFG rules: " + tag.rules.size());
		
	}
	
	
	private void initRule ( TAGTreeNode parent , List children )
	{
		TAGTreeNode[] rule = new TAGTreeNode[children.size()+1];
		rule[0] = parent;
		for ( int i = 0 ; i < children.size() ; i++ )
		{
			rule[i+1] = (TAGTreeNode) children.get(i);
		}
		rules.add(rule);
	}
	
	/**
	 * Builds CFG-style rules representing a TAG tree.
	 * This method should be called on a TAG tree, its root DOM node and on its
	 * TAG tree node, and will traverse all the nodes and add the corresponding 
	 * rules to the grammar.
	 * Return value is the foot node, if it is present. This way, the rule
	 * Foot -> Inverted_T can easily be created later.
	 * @param aNode Wrapper of the DOM Node to start the traversal.
	 */
	private TAGTreeNode initRulesFromNode ( TAGTree theTree , DOMNodeWrapper aNode , TAGTreeNode sameNode ) throws IncorrectGrammarFormatException
	{
		TAGTreeNode foot = null;
		String nodeType = aNode.getAttribute("type");
		if ( nodeType.equals("foot") )
			foot = sameNode;
		DOMNodeListWrapper children = aNode.getDirectChildrenElementsByTagName("node");
		if ( children.size() > 0 ) //else, no children -> no rules
		{
			List childNodeList = new ArrayList();
			List childDOMNodeList = new ArrayList();
			for ( int i = 0 ; i < children.size() ; i++ )
			{
				DOMNodeWrapper childNode = children.get(i);
				String childCat = childNode.getAttribute("cat");
				if ( childCat == null )
				{
					String childLex = childNode.getAttribute("lex");
					if ( childLex == null )
						throw new IncorrectGrammarFormatException( "Node lacks category and lexeme" );
					childCat = childLex;
				}
				if ( "".equals(childCat) )
				{
					//epsilon node
					childCat = "epsilon";
					
					//if ( "lex".equals(childNode.getAttribute("type")) )
					//	childCat = "epsilon";
					//else
					//{
						//debug output
					/*
						try
						{
							Document doc;
							doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
							org.w3c.dom.Node n = doc.importNode(childNode.getNode(),true);
							doc.appendChild(n);
							System.err.println("Node lacks category:");
							System.err.println(doc);
							Transformer xformer = TransformerFactory.newInstance().newTransformer();
							System.out.println("At tree " + currentTreeName + ": ");
							Result result = new StreamResult (System.out);
							xformer.transform(new DOMSource(doc),result);
						}
						catch ( Throwable t ){t.printStackTrace();}
					*/
						//throw new IncorrectGrammarFormatException ( "Node lacks category: " + childNode.getNode() + " - " + childNode.getNode().getClass() );

					//}
				}
				
				//new format
				boolean nullAdjunction = "no".equals(childNode.getAttribute("adj"));
				
				//old format
				nullAdjunction = nullAdjunction || "none".equals(childNode.getAttribute("type"));
				//nullAdjunction = nullAdjunction || "anchor".equals(childNode.getAttribute("type"));
				nullAdjunction = nullAdjunction || "subst".equals(childNode.getAttribute("type"));				
				
				boolean subsNode = "subst".equals(childNode.getAttribute("type"));
				
				//new format
				subsNode = subsNode || "subs".equals(childNode.getAttribute("type"));
				
				TAGTreeNode childTreeNode = 
					theTree.addNodeFor( SymbolTable.instance().symbolFor(childCat) , nullAdjunction , subsNode );
				
				boolean anchor = "anchor".equals(childNode.getAttribute("type"));
				if ( anchor )
					theTree.addAnchor( childTreeNode );
				
				//this has an empty implementation in TreeAdjoiningGrammar without features
				initFeatures ( childTreeNode , childNode );
				
				childNodeList.add(childTreeNode);
				childDOMNodeList.add(childNode);
			}
			//add rule
			initRule ( sameNode , childNodeList );
			//continue traversal: recursive calls for children
			for ( int i = 0 ; i < childNodeList.size() ; i++ )
			{
				DOMNodeWrapper domNode = (DOMNodeWrapper) childDOMNodeList.get(i);
				TAGTreeNode childNode = (TAGTreeNode) childNodeList.get(i);
				TAGTreeNode possibleFoot = initRulesFromNode ( theTree , domNode , childNode );
				if ( foot == null && possibleFoot != null )
					foot = possibleFoot;
			}
		}
		return foot;
	}
	
	
	//this is for debugging, remove it
	String currentTreeName;
	
	
	public void initTree(DOMNodeWrapper treeNode , String familyName) throws IncorrectGrammarFormatException
	{
		//initialize a TAG tree from its corresponding DOM node.
		String treeName = treeNode.getAttribute("name");
		currentTreeName = treeName;
		if ( treeName == null )
			throw new IncorrectGrammarFormatException ( "Unnamed tree" );
		
		DOMNodeListWrapper rootNodes = treeNode.getDirectChildrenElementsByTagName("node");
		if ( rootNodes.size() != 1 )
		{
			throw new IncorrectGrammarFormatException( rootNodes.size() + " root nodes at tree" );
		}
		DOMNodeWrapper rootNode = rootNodes.get(0);
		String rootCat = rootNode.getAttribute("cat");
		
		if ( rootCat == null )
		{
			String rootLex = rootNode.getAttribute("lex");
			if ( rootLex == null )
				throw new IncorrectGrammarFormatException( "Root node lacks category and lexeme" );
			rootCat = rootLex;
		}
		if ( "".equals(rootCat) )
		{
			//epsilon node
			rootCat = "epsilon";
		}
		
		//new format
		boolean nullAdjunction = "no".equals(rootNode.getAttribute("adj"));
		
		//old format
		nullAdjunction = nullAdjunction || "none".equals(rootNode.getAttribute("type"));
		//nullAdjunction = nullAdjunction || "anchor".equals(rootNode.getAttribute("type"));
		nullAdjunction = nullAdjunction || "subst".equals(rootNode.getAttribute("type"));
		nullAdjunction = nullAdjunction || "foot".equals(rootNode.getAttribute("type"));
		
		//new format
		boolean subsNode = "subs".equals(rootNode.getAttribute("type"));
		
		//old format
		subsNode = subsNode || "subst".equals(rootNode.getAttribute("type"));
		
		TAGTree theTree = new TAGTree ( this , treeName );
		
		TAGTreeNode realRoot = theTree.addNodeFor(SymbolTable.instance().symbolFor(rootCat) , nullAdjunction , subsNode);
		
		initFeatures ( realRoot , rootNode );
		
		//agregar regla T -> raiz
		List realRootList = new ArrayList();
		realRootList.add(realRoot);
		initRule ( theTree.getArtificialRoot(), realRootList );
		
		TAGTreeNode footNode = initRulesFromNode ( theTree , rootNode , realRoot );
		
		if ( footNode == null )
			theTree.setInitial(true);
		else
			theTree.setInitial(false);
			//throw new IncorrectGrammarFormatException ( "Tree " + treeName + " has no foot node");
		
		//agregar regla raiz -> T invertida
		if ( footNode != null )
		{
			TAGTreeNode artificialFoot = theTree.addNodeFor(SymbolTable.instance().symbolFor("Tfoot"));
			List invertedTList = new ArrayList();
			invertedTList.add(artificialFoot);
			initRule ( footNode , invertedTList );
		}
		
		//familia
		theTree.setFamily(familyName);
		
		trees.add(theTree);
		
	}
	
	public void initFromFile(File f) throws IOException, IncorrectGrammarFormatException 
	{
		
		try
		{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setValidating(true);
			factory.setNamespaceAware(true);
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document document = builder.parse( f );
			
			DOMNodeWrapper root = new DOMNodeWrapper ( document.getDocumentElement() );
			
			//axiom
			String axiomStr = root.getAttribute("axiom");
			if ( axiomStr == null )
				throw new IncorrectGrammarFormatException("tag node lacking attribute axiom");
			axiom = SymbolTable.instance().symbolFor(axiomStr);

			//family (trees) file
			String famFileStr = root.getAttribute("familyfile");
			File fTrees = new File(f.getParentFile(),famFileStr);
			Document docTrees = builder.parse(fTrees);
			DOMNodeWrapper rootTrees = new DOMNodeWrapper ( docTrees.getDocumentElement() );
			
			//lemma file
			String lemmaFileStr = root.getAttribute("lemmafile");
			File fLemmas = new File(f.getParentFile(),lemmaFileStr);
			Document docLemmas = builder.parse(fLemmas);
			DOMNodeWrapper rootLemmas = new DOMNodeWrapper ( docLemmas.getDocumentElement() );
			
			//lex file
			String lexFileStr = root.getAttribute("lexfile");
			File fLex = new File(f.getParentFile(),lexFileStr);
			Document docLex = builder.parse(fLex);
			DOMNodeWrapper rootLex = new DOMNodeWrapper ( docLex.getDocumentElement() );
			
			//tree families
			DOMNodeListWrapper famNodes = rootTrees.getElementsByTagName("family");
			for ( int i = 0 ; i < famNodes.size() ; i++ )
			{
				String familyName = famNodes.get(i).getAttribute("name");
				DOMNodeListWrapper treeNodes = famNodes.get(i).getElementsByTagName("tree");
				for ( int j = 0 ; j < treeNodes.size() ; j++ )
				{
					DOMNodeWrapper treeNode = treeNodes.get(j);
					initTree ( treeNode , familyName );
				}
			}
			
			//single trees
			DOMNodeListWrapper singNodes = rootTrees.getElementsByTagName("single-tree");
			for ( int i = 0 ; i < singNodes.size() ; i++ )
			{
				DOMNodeListWrapper treeNodes = singNodes.get(i).getElementsByTagName("tree");
				for ( int j = 0 ; j < treeNodes.size() ; j++ )
				{
					DOMNodeWrapper treeNode = treeNodes.get(j);
					initTree ( treeNode , null );
				}
			}
			
			
			
		}
		catch ( ParserConfigurationException pce )
		{
			throw new IncorrectGrammarFormatException(pce);
		}
		catch ( SAXException se )
		{
			se.printStackTrace();
			throw new IncorrectGrammarFormatException(se);
		}
		
		
		
		// TODO Auto-generated method stub
		
	}

	public Element[] getEntity(int i) 
	{
		return (Element[]) rules.get(i);
	}

	public int size() 
	{
		return rules.size();
	}

	
	
	
}
