/*
 * Created on 12-ago-2004 13:17:47
 *
 */
package element;

/**
 * <p>Elemento que representa una posicin de la cadena de entrada. Estas posiciones
 * podrn variar de 0 a n, siendo n la longitud de la cadena. Ntese que la clase
 * StringPosition es una herencia por restriccin de la clase IntElement de los elementos
 * enteros.</p>
 * <p>El sentido de tener esta clase en lugar de utilizar directamente IntElement en
 * todas partes es que las StringPosition tienen la ventaja de que estn acotadas, y por
 * lo tanto pueden implementar ArrayIndexable.</p>
 * <p>Created 12-ago-2004 13:17:47</p>
 * @author carlos
 */
public class StringPosition extends IntElement implements HashIndexable, ArrayIndexable
{

    /**El mximo valor que puede tomar una de estas posiciones.*/
    private static int maxValue;
    
    /**
     * @param value Valor que tomar esta posicin de la cadena.
     * @param ground Si el valor es ground o no.
     */
    public StringPosition ( int value, boolean ground )
    {
        super ( value, ground );
    }

    /**
     * @see element.ArrayIndexable#getArrayIndex()
     */
    public int getArrayIndex ()
    {
        return value;
    }
    
	/**
	 * Devuelve el propio objeto StringPosition.
	 * @see element.HashIndexable#getHashKey()
	 */
	public Object getHashKey() 
	{
		return this;
	}
	
    /**
     * Este mtodo se debe llamar cuando se conozca la longitud de la cadena de entrada,
     * estableciendo el mximo valor que puede tomar una StringPosition. De ah se deducir
     * la longitud de los arrays para ndices sobre posiciones.
     */
    public static void setMaxValue( int val )
    {
        maxValue = val;
        //inicializamos el array que evita crear instancias de Integer
        commonIntegerInstances = new Integer[val+1];
        for ( int i = 0 ; i < commonIntegerInstances.length ; i++ )
        	commonIntegerInstances[i] = new Integer(i);
    }
    
    /**
     * @see element.ArrayIndexable
     * @return Longitud del array para indexar las posiciones de la cadena de entrada.
     */
    public static int getArrayLength()
    {
        return maxValue + 1;
    }
    
    /**
     * Mximo valor que puede tomar una posicin de cadena de entrada. Depender de la
     * frase, y nos da su longitud (n).
     * @return Longitud de la frase, o mximo valor de una posicin de cadena
     * de entrada.
     */
    public static int getMaxValue()
    {
    	return maxValue;
    }
    
    private static int valuesForNonGround = 0;
    private static java.util.HashMap nonGroundHash = new java.util.HashMap(); //keys: String
    private static java.util.HashMap groundHash = new java.util.HashMap(); //keys: Integer
    
	/**
	 * Mtodo de parsing para StringPositions, nos da la posicin asociada a una cadena dada.
	 * (non ground)
	 * @return Smbolo correspondiente a la cadena dada.
	 */
	public static StringPosition nonGroundFromString(String s)
	{
	    if ( nonGroundHash.get(s) != null )
	        return (StringPosition) nonGroundHash.get(s);
	    else
	    {
	        //nueva
	        valuesForNonGround++;
	        StringPosition nueva = new StringPosition(valuesForNonGround,false);
	        nonGroundHash.put(s,nueva);
	        return nueva;
	    }
	}
	
	/**
	 * Obtiene un objeto StringPosition constante (ground) con el valor dado.
	 * Ntese que estos objetos se comparten, es decir, si se obtiene la constante
	 * para el mismo valor varias veces se devolver el mismo objeto.
	 * @param i Valor entero que toma la posicin de la cadena de entrada.
	 * @return Objeto StringPosition canonicalizado que representa ese valor.
	 */
	public static StringPosition groundFromValue(Integer i)
	{
		StringPosition sp = (StringPosition) groundHash.get(i);
	    if ( sp != null )
	    {
	        return sp;
	    }
	    else
	    {
	        //nueva
	        StringPosition nueva = new StringPosition(i.intValue(),true);
	        groundHash.put(i,nueva);
	        return nueva;
	    }
	}
		
	//array que ahorra a groundFromValue(int) crear instancias de enteros.
	protected static Integer[] commonIntegerInstances; 
	
	/**
	 * Hace lo mismo que groundFromValue(Integer), es decir, construir una
	 * constante StringPosition a partir de su valor expresado como entero.
	 * La diferencia es que trabaja con el tipo base en lugar de con un wrapper.
	 * @param i Valor entero de la constante StringPosition que se desea obtener.
	 * @return Constante StringPosition que tiene ese valor, igual a las que se
	 * hayan obtenido con la misma llamada y parmetro (i.e. el mismo objeto).
	 */
	public static StringPosition groundFromValue ( int i )
	{
		if ( i<= maxValue && maxValue != 0 )
			return groundFromValue ( commonIntegerInstances[i] );
		else
			return groundFromValue ( new Integer(i) );
	}
	
	/**
	 * Obtiene un objeto StringPosition ground (constante) a partir de una cadena
	 * que represente su valor. La cadena tiene que representar un nmero entero,
	 * por ejemplo "1".
	 * @param sval Cadena que representa un valor constante asociable a una
	 * posicin de cadena de entrada.
	 * @return Objeto StringPosition con el valor entero de esa cadena.
	 */
	public static StringPosition groundFromString(String sval)
	{
	    return groundFromValue(Integer.valueOf(sval));
	}
	
	/**
	 * En el caso de las StringPosition ground, devolvemos una cadena con su valor
	 * (p.ej. "3"), y en el caso de las no-ground, devolvemos un entero prefijado con
	 * una i (i1, i2, etc.)
	 */
	public String toString()
	{
	    if ( isGround() )
	        return ""+value;
	    else
	        return "i"+value;
	}
	
	/**
	 * Devuelve el cdigo Java para que las posiciones ground aparezcan en expresiones.
	 */
	public String getExpressionCode()
	{
	    if ( ground )
	        return "StringPosition.groundFromValue("+value+")";
	    else
	        return null;
	}
	
	/**
	 * En este caso, al contrario que en los IntElements generales, equals hace
	 * por dentro una comparacin de identidad.
	 */
	public boolean equals ( Object o )
	{
		return ( o != null && this == o );
	}
	
	public NullableStringPosition nullable()
	{
		if ( this.isGround() )
			return NullableStringPosition.groundFromValue(false,value);
		else
			return null;
	}
	
	
}
