package element.fs;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Esta clase representa una expresin booleana de tipo Keep-vars(fs1;fs2)
 * que nos devuelve el resultado de borrar de fs2 las variables que no
 * aparezcan en fs1.
 * @author Carlos
 */
public class KeepVarsExpression extends ExpressionElement 
{

	public KeepVarsExpression(Element[] arguments)
	{
		super( arguments );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return PlainFeatureStructure.class;
	}
	
	 /**
     * La expresin Adjoinable se resuelve al cdigo Java que llama a un mtodo
     * que determina si un rbol es adjuntable en un nodo.
     */
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("");
        result.append(memberNames[0]);
        for ( int i = 1 ; i < memberNames.length ; i++ )
        {
        	result.append(".keepVariables( ");
        	result.append(memberNames[i]);
        	result.append(" )");
        }
        return result.toString();
	}
	
	public static KeepVarsExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() < 2 )
			{
				throw new ElementParseException("KeepVars expression taking " + expressionArgs.size() + " arguments when 2 or more are expected");
			}
			
			Element[] arguments = new Element[expressionArgs.size()];
			for ( int i = 0 ; i < arguments.length ; i++ )
			{
				arguments[i] = ElementParser.getCurrentInstance().parse((String)expressionArgs.get(i));
			}
			
			return new KeepVarsExpression ( arguments );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
