package element.fs;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.TreeSet;

import element.HashIndexable;
import element.SimpleElement;
import element.tag.TAGTree;


/**
 * New, Optimized Plain XTAG-Type Feature Structure Implementation
 */


public class PlainFeatureStructure extends SimpleElement implements HashIndexable
{

	/*
	 * Data Structures
	 */
	private List trees = new ArrayList(); //of TAGTree
	private List variableIndexesToVariables = new ArrayList(); //List of Array of Variable.
									 //varsToFeatureLists.get(treeIndex)[varNumberIndex] = a variable name
	private List featureIndexesToVariables = new ArrayList();
	private List featureIndexesToValues = new ArrayList();
	
	class Variable
	{
		private List featureIndexList = new ArrayList();
		
		private TAGTree tree;
		private int index;
		
		private String value;
		
		public Variable ( TAGTree tree , int index )
		{
			this.tree = tree;
			this.index = index;
		}
		
		public void setValue ( String value )
		{
			this.value = value;
		}
		
		public String getValue()
		{
			return value;
		}
		
		public boolean equals ( Object o )
		{
			//equality is now full
			
			if ( !(o instanceof Variable) ) return false;
			if ( o == null ) return false;
			Variable v = (Variable) o;
			/*
			return this.featureIndexList.equals(v.featureIndexList);
			*/
			if ((this.tree != v.tree) || (this.index != v.index)) return false;
			
			if ( this.value == null )
			{
				if ( v.value != null ) return false;
			}
			else if ( v.value == null )
			{
				if ( this.value != null ) return false;
			}
			else		
				if (!this.value.equals(v.value)) return false;
			
			return ( this.featureIndexList.equals(v.featureIndexList) );
		}
		public int hashCode()
		{
			int hash = 1;
			/*
			for ( int i = 0 ; i < featureIndexList.size() ; i++ )
			{
				hash = hash*37 + ((Integer)featureIndexList.get(i)).intValue();
			}
			*/
			hash = hash*37+tree.hashCode();
			hash = hash*37+index;
			hash = hash*37+value.hashCode();
			return hash;
		}
		public Object clone()
		{
			Variable v = new Variable(tree,index);
			v.featureIndexList = (List) ((ArrayList)this.featureIndexList).clone();
			v.value = this.value;
			return v;
		}
		public String toString()
		{
			StringBuffer sb = new StringBuffer();
			sb.append("<VAR ");
			sb.append(index);
			sb.append("@");
			sb.append(tree.getName());
			
			if ( featureIndexList.size() > 0 )
			{
				sb.append("#");
				sb.append("LINK: ");
				Iterator it = featureIndexList.iterator();
				while ( it.hasNext() )
				{
					sb.append(indexToFeatureName(((Integer)it.next()).intValue()));
					if ( it.hasNext() ) sb.append(", ");
				}
			}
			if ( value != null )
			{
				sb.append("#");
				sb.append("VAL: ");
				sb.append(value);
			}
			sb.append(">");
			return sb.toString();
		}
	}
	
	private static Map featureNameToIndexMap = new HashMap();
	private static List indexToFeatureNameList = new ArrayList();
	private static int lastFeatureIndexUsed = -1;
	
	private static int addFeatureMapping ( String feature )
	{
		lastFeatureIndexUsed++;
		featureNameToIndexMap.put(feature,new Integer(lastFeatureIndexUsed));
		indexToFeatureNameList.add(feature);
		return lastFeatureIndexUsed;
	}
	
	private int featureNameToIndex ( String feature )
	{
		Integer ind = (Integer) featureNameToIndexMap.get(feature);
		if ( ind != null ) return ind.intValue();
		else return addFeatureMapping ( feature );
	}
	
	private String indexToFeatureName ( int index )
	{
		return (String) indexToFeatureNameList.get(index);
	}
	
	
	//test method
	public static void main ( String[] args )
	{
		
		
		PlainFeatureStructure pfs1 = new PlainFeatureStructure(true);
		PlainFeatureStructure pfs2 = new PlainFeatureStructure(true);
		PlainFeatureStructure pfs3 = new PlainFeatureStructure(true);
		PlainFeatureStructure pfs4 = new PlainFeatureStructure(true);
		PlainFeatureStructure pfs5 = new PlainFeatureStructure(true);
		int AGR = PlainFeatureStructure.addFeatureMapping("agr");
		int AGRCPY = PlainFeatureStructure.addFeatureMapping("agrCopy");
		int NUMBERS = PlainFeatureStructure.addFeatureMapping("numbers");
		int NUMEROS = PlainFeatureStructure.addFeatureMapping("numeros");
		pfs1.setFeatureValue(AGR,"sing");
		pfs2.setFeatureValue(AGR,"plur");
		System.out.println(PlainFeatureStructure.unifies(pfs1,pfs2));
		System.out.println(PlainFeatureStructure.unifies(pfs1,pfs3));
		TAGTree t = new TAGTree(null,"TestTree");
		/*
		pfs3.trees.add(t);
		Variable[] vars = new Variable[1];
		Variable v = pfs3.new Variable(t,1);
		vars[0] = v;
		pfs3.variableIndexesToVariables.add(vars);
		pfs3.linkFeatureToVariable(AGR,v);
		pfs3.linkFeatureToVariable(AGRCPY,v);
		System.out.println(pfs1);
		System.out.println(pfs3);
		System.out.println(PlainFeatureStructure.unifies(pfs3,pfs1));
		System.out.println(pfs1);
		System.out.println(pfs3);
		System.out.println(PlainFeatureStructure.unify(pfs3,pfs1));
		System.out.println(pfs1);
		System.out.println(pfs3);
		System.out.println(PlainFeatureStructure.unify(pfs1,pfs3));
		PlainFeatureStructure pfs6 = new PlainFeatureStructure(true);
		PlainFeatureStructure pfs7 = new PlainFeatureStructure(true);
		PlainFeatureStructure pfs8 = new PlainFeatureStructure(true);
		pfs6.setFeatureValue(NUMBERS,"1/2/3/4/5/6");
		pfs7.setFeatureValue(NUMBERS,"1/3/5/7/9/11");
		pfs7.trees.add(t);
		Variable[] vars2 = new Variable[1];
		Variable v2 = pfs7.new Variable(t,2);
		vars2[0] = v2;
		pfs7.variableIndexesToVariables.add(vars2);
		System.out.println("LINK");
		pfs7.linkFeatureToVariable(NUMBERS,v2);
		System.out.println("ZELDA");
		System.out.println(PlainFeatureStructure.unifies(pfs6,pfs7));
		System.out.println(PlainFeatureStructure.unify(pfs6,pfs7));
		pfs3.linkFeatureToVariable(AGRCPY,v);
		pfs8.trees.add(t);
		pfs8.variableIndexesToVariables.add(vars2);
		pfs8.linkFeatureToVariable(NUMEROS,v2);
		System.out.println("6:"+pfs6);
		System.out.println("7:"+pfs7);
		System.out.println("8:"+pfs8);
		System.out.println("7 u 6:"+PlainFeatureStructure.unify(pfs7,pfs6));
		System.out.println("8 u (7 u 6):"+unify(pfs8,unify(pfs7,pfs6)));
		System.out.println("(7 u 6) u 8:"+unify(unify(pfs7,pfs6),pfs8));
		System.out.println("Equality:"+(unify(unify(pfs7,pfs6),pfs8)).equals(unify(pfs8,unify(pfs7,pfs6))));
		*/
		
		PlainFeatureStructure p1 = new PlainFeatureStructure(true);
		p1.trees.add(t);
		Variable x5 = p1.new Variable(t,5);
		Variable[] vars5 = new Variable[] { x5 };
		p1.variableIndexesToVariables.add(vars5);
		p1.linkFeatureToVariable(0,x5); //0 = mode
		
		PlainFeatureStructure p2 = new PlainFeatureStructure(true);
		p2.trees.add(t);
		Variable x6 = p2.new Variable(t,6);
		Variable[] vars6 = new Variable[] { null,x6 };
		p2.variableIndexesToVariables.add(vars6);
		p2.setVariableValue(x6,"nom");
				
		PlainFeatureStructure p3 = new PlainFeatureStructure(true);
		//p3.setFeatureValue(0,"ind");
		
		PlainFeatureStructure p4 = new PlainFeatureStructure(true);
		p4.trees.add(t);
		x6 = p4.new Variable(t,6);
		vars6 = new Variable[] { null,x6 };
		p4.variableIndexesToVariables.add(vars6);
		p4.linkFeatureToVariable(0,x6); //0 = mode
		
		System.out.println("A: " + p1);
		System.out.println("B: " + unify(p1,p2));
		System.out.println("C: " + unify(unify(p1,p2),p3));
		System.out.println("D: " + unify(unify(unify(p1,p2),p3),p4));
		
		System.out.println("A2: " + p4);
		System.out.println("B2: " + unify(p4,p3));
		System.out.println("C2: " + unify(unify(p4,p3),p2));
		System.out.println("D2: " + unify(unify(unify(p4,p3),p2),p1));
		
		System.out.println("A3: " + p1);
		System.out.println("B3: " + unify(p1,p3));
		System.out.println("C3: " + unify(unify(p1,p3),p2));
		System.out.println("D3: " + unify(unify(unify(p1,p3),p2),p4));
		
		System.out.println("A4: " + p2);
		System.out.println("B4: " + unify(p2,p3));
		System.out.println("C4: " + unify(unify(p2,p3),p4));
		System.out.println("D4: " + unify(unify(unify(p2,p3),p4),p1));
		
		
		/*
		pfs8.linkFeatureToVariable("numeros","X1");
		System.out.println("6:"+pfs6);
		System.out.println("7:"+pfs7);
		System.out.println("8:"+pfs8);
		System.out.println("7 u 6:"+pfs7.unify(pfs6));
		System.out.println("7 u 6:"+unify(pfs7,pfs6));
		System.out.println("8 u (7 u 6):"+pfs8.unify(pfs7.unify(pfs6)));
		System.out.println("(7 u 6) u 8:"+pfs7.unify(pfs6).unify(pfs8));
		System.out.println("(7 u 6) u 8:"+unify(unify(pfs7,pfs6),pfs8));
		System.out.println("Equality: " +pfs8.unify(pfs7.unify(pfs6)).equals(pfs7.unify(pfs6).unify(pfs8)));
		*/
	}
		
	public int hashCode()
	{
		//now variable-based instead of feature-based
		int hash = 1;
		hash = hash * 31 + ((this.trees == null)? 0 : this.trees.hashCode());
		if ( this.variableIndexesToVariables != null )
		{
			for ( int i = 0 ; i < this.variableIndexesToVariables.size() ; i++ )
			{
				Variable[] ar = (Variable[]) this.variableIndexesToVariables.get(i);
				for ( int j = 0 ; j < ar.length ; j++ )
				{
					if ( ar[j] != null )
						hash = hash * 31 + ar[j].hashCode();
				}
			}
		}
		
		
		//enlarge array if necessary
		/*
		while ( featureIndexesToValues.size() < PlainFeatureStructure.lastFeatureIndexUsed+1 )
			featureIndexesToValues.add(null);
		while ( featureIndexesToVariables.size() < PlainFeatureStructure.lastFeatureIndexUsed+1 )
			featureIndexesToVariables.add(null);		
		*/
		/*
		int hash = 1;
		*/
		/*
		hash = hash * 31 + ((this.trees == null)? 0 : this.trees.hashCode());
		if ( this.variableIndexesToVariables != null )
		{
			for ( int i = 0 ; i < this.variableIndexesToVariables.size() ; i++ )
			{
				Variable[] ar = (Variable[]) this.variableIndexesToVariables.get(i);
				for ( int j = 0 ; j < ar.length ; j++ )
				{
					if ( ar[j] != null )
						hash = hash * 31 + ar[j].hashCode();
				}
			}
		}
		*/
		//hash = hash * 31 + ((this.variableIndexesToVariables == null)? 0 : this.variableIndexesToVariables.hashCode());
		//hash = hash * 31 + ((this.featureIndexesToVariables == null)? 0 : this.featureIndexesToVariables.hashCode());
		//hash = hash * 31 + ((this.featureIndexesToValues == null)? 0 : this.featureIndexesToValues.hashCode());
		return hash;
	}
	
	public Object getHashKey()
	{
		return this;
	}
	
	public boolean equals ( Object o )
	{
		
		if ( ! ( o instanceof PlainFeatureStructure ) ) return false;
		if ( !ground ) return ( o != null && this == o );		
		PlainFeatureStructure another = (PlainFeatureStructure)o;
		
		if ( this.variableIndexesToVariables == null )
		{
			//another's variable structure must be null or empty
			if ( another.variableIndexesToVariables == null ) return true;
			for ( int i = 0 ; i < another.variableIndexesToVariables.size() ; i++ )
			{
				Variable[] ar = (Variable[]) another.variableIndexesToVariables.get(i);
				for ( int j = 0 ; j < ar.length ; j++ )
				{
					if ( ar[j] != null ) return false;
				}
			}
			return true;
		}
		else if ( another.variableIndexesToVariables == null )
		{
			//symmetric operation
			if ( this.variableIndexesToVariables == null ) return true;
			for ( int i = 0 ; i < this.variableIndexesToVariables.size() ; i++ )
			{
				Variable[] ar = (Variable[]) this.variableIndexesToVariables.get(i);
				for ( int j = 0 ; j < ar.length ; j++ )
				{
					if ( ar[j] != null ) return false;
				}
			}
			return true;
		}
		else
		{
			//complex case
			for ( int i = 0 ; i < this.variableIndexesToVariables.size() ; i++ )
			{
				TAGTree thisTree = (TAGTree) this.trees.get(i);
				int anotherTreeInd = another.trees.indexOf(thisTree);
				if ( anotherTreeInd < 0 ) return false; //trees are not the same
				Variable[] cmp1 = (Variable[]) this.variableIndexesToVariables.get(i);
				Variable[] cmp2 = (Variable[]) another.variableIndexesToVariables.get(anotherTreeInd);
				int limit = Math.max(cmp1.length , cmp2.length);
				//compare arrays cmp1, cmp2
				for ( int j = 0 ; j < limit ; j++ )
				{
					if ( j >= cmp1.length || cmp1[j] == null ) //empty (nonpresent) variable
					{
						if ( j < cmp2.length && cmp2[j] != null ) return false;
					}
					else //nonempty variable
					{
						if ( j >= cmp2.length || cmp2[j] == null  ) return false;
						if ( !cmp1[j].equals(cmp2[j]) ) return false;
					}
				}
			}
			return true;
		}
		
		

		
		
		
		
		
		
		/*
		return this.featureToValueMap.equals(another.featureToValueMap)
		&& this.featureToVarNameMap.equals(another.featureToVarNameMap);
		*/
		
		/*
		OPTIMIZED EQUALS order of operations based on a test run with the following data:
		
		total 30,000,000 equals executed
		
		this.featureToValueMap.size() == another.featureToValueMap.size():
		fails 20,180,526 times
		
		this.featureToVarNameMap.size() == another.featureToVarNameMap.size():
		fails 4,361,175 times 
		
		this.featureToValueMap.equals(another.featureToValueMap):
		fails 21,606,377 times
		
		this.featureToVarNameMap.equals(another.featureToVarNameMap):
		fails 5,928,652 times
		
		(of a total of 22,653,834 false results, the other 7,346,166 were true)
		*/
		
		
		//enlarge array if necessary
		/*
		while ( featureIndexesToValues.size() < PlainFeatureStructure.lastFeatureIndexUsed+1 )
			featureIndexesToValues.add(null);
		while ( featureIndexesToVariables.size() < PlainFeatureStructure.lastFeatureIndexUsed+1 )
			featureIndexesToVariables.add(null);
		while ( another.featureIndexesToValues.size() < PlainFeatureStructure.lastFeatureIndexUsed+1 )
			another.featureIndexesToValues.add(null);
		while ( another.featureIndexesToVariables.size() < PlainFeatureStructure.lastFeatureIndexUsed+1 )
			another.featureIndexesToVariables.add(null);
		*/
		
		//if ( this.featureIndexesToValues.size() != another.featureIndexesToValues.size() )
		//	return false;
		/*
		if ( !this.featureIndexesToValues.equals(another.featureIndexesToValues) )
		{
			//System.out.print("%1%");
			return false;
		}
		*/
		
		//redundant
		/*
		if ( !this.trees.equals(another.trees) )
		{
			//System.out.print("%2%");
			return false;
		}
		*/
		
		/*
		if ( this.variableIndexesToVariables.size() != another.variableIndexesToVariables.size() ) 
		{
			//System.out.print("%3%");
			return false;
		}
		*/
		
		//redundant with featureIndexesToVariables
		/*
		for ( int i = 0 ; i < this.variableIndexesToVariables.size() ; i++ )
		{
			Variable[] oneArray = (Variable[]) this.variableIndexesToVariables.get(i);
			Variable[] anotherArray = (Variable[]) another.variableIndexesToVariables.get(i);			
			if ( oneArray.length != anotherArray.length )
			{
				//System.out.print("%4%");
				return false;
			}
			for ( int j = 0 ; j < oneArray.length ; j++ )
			{
				if ( oneArray[j] == null )
				{
					if ( anotherArray[j] != null ) return false;
				} 
				else if (!oneArray[j].equals(anotherArray[j])) 
				{
					//System.out.print("%5%");
					return false;
				}
			}
		}
		*/
		/*
		for ( int i = 0 ; i < this.featureIndexesToVariables.size() ; i++ )
		{
			List oneEntry = (List) featureIndexesToVariables.get(i);
			List anotherEntry = (List) another.featureIndexesToVariables.get(i);
			if ( oneEntry != null  && oneEntry.size() > 0 )
			{
				if ( !oneEntry.equals(anotherEntry) ) return false;
			}
			else if ( anotherEntry != null && anotherEntry.size() > 0 ) return false;
		}

			
		
		return true;
		*/
	}
		
	public String toString()
	{
		if ( ground )
		{
			StringBuffer sb = new StringBuffer();
			//Iterator it = this.features();
			sb.append("[");
			boolean wroteSomething = false;
			
			for ( int i = 0 ; i < variableIndexesToVariables.size() ; i++ )
			{
				Variable[] varAr = (Variable[]) variableIndexesToVariables.get(i);
				for ( int j = 0 ; j < varAr.length ; j++ )
				{
					if ( wroteSomething  )
						sb.append(" | ");
					Variable v = varAr[j];
					if ( v != null )
					{
						sb.append(v);
						wroteSomething = true;
					}
				}
			}
			
//			/*
			for ( int i = 0 ; i < Math.max ( featureIndexesToValues.size() , featureIndexesToVariables.size() ) ; i++ )
			{
				String feature = indexToFeatureName(i);
				String val = null;
				if ( i < featureIndexesToValues.size() )
					val = (String) featureIndexesToValues.get(i);
				List vars = null;
				if ( i < featureIndexesToVariables.size() )
					vars = (List) featureIndexesToVariables.get(i);
				//System.out.println("Vars: " + vars);
				if ( val != null || (vars != null && vars.size() > 0) )
				{
					if ( wroteSomething  )
						sb.append(" | ");
					sb.append(feature);
					sb.append(" = ");
					if ( val != null )
						sb.append(val);
					if ( vars != null )
					{
						for ( int j = 0 ; j < vars.size() ; j++ )
						{
							Variable var = (Variable) vars.get(j);
							sb.append(var);
						}
					}
					wroteSomething = true;
				}
			}
			sb.append("]");
//			*/
			
			//sb.append("]");
			
			return new String(sb);
		}
		else
		{
			return "ngfs" + super.hashCode();
		}
	}
	
	//private Map featureToVarNameMap = new TreeMap();
	//private Map featureToValueMap = new TreeMap();
	//private Map varNameToFeatureMap = new HashMap(); //of List
	
	
	private boolean ground;
	
	private static java.util.HashMap nonGroundHash = new java.util.HashMap(); //keys: String
	

	public boolean isGround()
	{
		return ground;
	}
	
	/*
	private static int lastNonGroundInstance = -1;
	private int nonGroundInstanceNumber = 0;
	*/
	
	public static PlainFeatureStructure nonGroundFromString(String s)
	{
	    if ( nonGroundHash.get(s) != null )
	    {
	    	PlainFeatureStructure pfs = (PlainFeatureStructure) nonGroundHash.get(s);
	    	return pfs;
	    }
	    else
	    {
	        //nueva
	        PlainFeatureStructure nueva = new PlainFeatureStructure(false);
	        /*
	        lastNonGroundInstance++;
	        nueva.nonGroundInstanceNumber = lastNonGroundInstance;
	        */
	        nonGroundHash.put(s,nueva);
	        return nueva;
	    }
	}
	
	private static PlainFeatureStructure emptyInstance = new PlainFeatureStructure ( true );
	
	public static PlainFeatureStructure getEmptyInstance()
	{
		return emptyInstance;
	}
	
	public static PlainFeatureStructure emptyFromString ( String s )
	{
		return emptyInstance;
	}
	
	/**
	 * Devuelve el cdigo Java para que las estructuras de rasgos ground
	 * aparezcan en expresiones. Slo soporta la vaca.
	 */
	public String getExpressionCode()
	{
	    if ( ground )
	    {
	    	if ( !trees.isEmpty() || ! featureIndexesToValues.isEmpty() || !variableIndexesToVariables.isEmpty() || !featureIndexesToVariables.isEmpty() )
	    	{
	    		System.err.println("[ERROR] The empty feature structure is the only FS supported by PlainFeatureStructure.getExpressionCode()");
	    	}
	    	return "element.fs.PlainFeatureStructure.getEmptyInstance()";
	    }
	    else
	        return null;
	}
	
	public PlainFeatureStructure ( boolean ground ) 
	{
		this.ground = ground;
	}
	
	public PlainFeatureStructure ( PlainFeatureStructure toCloneFrom )
	{
		//newer, faster implementation of the copy-constructor
		this.ground = toCloneFrom.ground;
		
		//a shallow copy is OK in featureIndexesToValues for it only stores strings.
		this.featureIndexesToValues = (List) ((ArrayList)toCloneFrom.featureIndexesToValues).clone();
		
		//shallow copy of trees list (stores tree references, needn't clone the tree)
		this.trees = (List) ((ArrayList)toCloneFrom.trees).clone();
		
		//init featureIndexesToVariables to list of nulls, we'll fill it
		//when copying variableIndexesToVariables
		this.featureIndexesToVariables = new ArrayList ( toCloneFrom.featureIndexesToVariables.size() );
		for ( int i = 0 ; i < toCloneFrom.featureIndexesToVariables.size() ; i++ )
			featureIndexesToVariables.add(null);
		
		//deep copy of variableIndexesToVariables
		//and fill featureIndexesToVariables
		for ( int i = 0 ; i < toCloneFrom.variableIndexesToVariables.size() ; i++ )
		{
			Variable[] thisArray = (Variable[]) toCloneFrom.variableIndexesToVariables.get(i);
			Variable[] newArray = new Variable[thisArray.length];
			for ( int j = 0 ; j < thisArray.length ; j++ )
			{
				if ( thisArray[j] != null )
				{
					Variable newVar = (Variable) thisArray[j].clone();
					newArray[j] = newVar;
					List featIndexList = newVar.featureIndexList;
					for ( int k = 0 ; k < featIndexList.size() ; k++ )
					{
						Integer featIndex = (Integer) featIndexList.get(k);
						
						//enlarge array if necessary
						while ( featureIndexesToVariables.size() < featIndex.intValue()+1 )
							featureIndexesToVariables.add(null);
						
						List toAddTo = (List)(featureIndexesToVariables.get(featIndex.intValue()));
						
						if ( toAddTo == null )
						{
							toAddTo = new ArrayList();
							featureIndexesToVariables.set(featIndex.intValue(),toAddTo);
						}
						toAddTo.add(newVar);
					}
				}
			}
			this.variableIndexesToVariables.add(newArray);
		}		
	}
	
	/*
	public Iterator features()
	{
		Set one = featureToValueMap.keySet();
		return one.iterator();
	}
	*/
	
	public boolean isFeatureLinked ( int feature )
	{
		List l = (List) featureIndexesToVariables.get(feature);
		return ( l != null && !l.isEmpty() );
	}
	
	public boolean isFeatureLinked ( String feature )
	{
		return isFeatureLinked ( featureNameToIndex(feature) );
		
	}
	
	public boolean isFeatureDefined ( int feature )
	{
		return featureIndexesToValues.get(feature) != null;
	}
	
	public boolean isFeatureDefined ( String feature )
	{
		return isFeatureDefined ( featureNameToIndex(feature) );
	}
	
	/*
	public boolean isVariableDefined ( String variable )
	{
		List feats = (List) varNameToFeatureMap.get(variable);
		if ( feats == null || feats.size() < 1 ) //variable doesn't exist
			return false;
		else
		{
			String aLinkedFeature = (String) feats.get(0);
			return isFeatureDefined ( aLinkedFeature );
		}
	}
	*/
	
	public String getFeatureValue ( int feature )
	{
		//enlarge array if necessary
		while ( featureIndexesToValues.size() < feature+1 )
			featureIndexesToValues.add(null);
		
		return (String) featureIndexesToValues.get(feature);
	}
	
	public String getFeatureValue ( String feature )
	{
		return getFeatureValue ( featureNameToIndex(feature) );
	}
	
	/*
	public String getFeatureVariable ( String feature )
	{
		return (String) featureIndexesToValues.get(feature);
	}
	*/
	
	public String getVariableValue ( Variable variable )
	{
		/*
		List l = variable.featureIndexList;
		if ( l == null || l.size() == 0 ) return null;
		int index = ((Integer)l.get(0)).intValue();
		return (String) featureIndexesToValues.get(index);
		*/
		return variable.getValue();
	}
	
	/*
	public String getVariableValue ( String variable )
	{
		List feats = (List) varNameToFeatureMap.get(variable);
		if ( feats == null || feats.size() < 1 ) //variable doesn't exist
			return null;
		else
		{
			String aLinkedFeature = (String) feats.get(0);
			return getFeatureValue(aLinkedFeature);
		}
	}
	*/
	
	/**
	 * Usage example: given two strings "a/b/d/e/f/h" and "b/z/g/e/f",
	 * this would return a string "b/e/f"
	 */
	private static String disjunctionIntersection ( String s1 , String s2 )
	{
		String sep = "/";
		if ( s1.indexOf(sep) < 0 && s2.indexOf(sep) < 0 && !s1.equals(s2) ) return "";
		Set oneSet = new TreeSet();
		StringTokenizer st = new StringTokenizer(s1,sep);
		while ( st.hasMoreTokens() ) oneSet.add(st.nextToken());
		StringBuffer result = new StringBuffer();
		StringTokenizer st2 = new StringTokenizer(s2,sep);
		boolean nonEmptyResult = false;
		while ( st2.hasMoreTokens() )
		{
			String tok = st2.nextToken();
			if ( oneSet.contains(tok) )
			{
				if ( nonEmptyResult )
					result.append(sep);
				nonEmptyResult = true;
				result.append(tok);
			}	
		}
		return new String(result);
	}
	
	public boolean linkFeatureToVariable ( int featIndex , Variable var )
	{
		//enlarge array if necessary
		while ( featureIndexesToValues.size() < featIndex+1 )
			featureIndexesToValues.add(null);
		//enlarge array if necessary
		while ( featureIndexesToVariables.size() < featIndex+1 )
			featureIndexesToVariables.add(null);
		
		
		String varValue = getVariableValue(var);
		String featValue = getFeatureValue(featIndex);
		if ( featValue != null && varValue != null && !featValue.equals(varValue) )
		{
			//feature's current value doesn't exactly match variable's current value.
			//but they may be intersecting disjunctions.
			String intersect = disjunctionIntersection ( featValue , varValue );
			if ( intersect.length() < 1 )
				return false; //values don't match
			else
			{
				varValue = intersect;
				featValue = intersect;
				//... and set the values!
			}
		}
		else if ( featValue == null && varValue != null )
		{
			featValue = varValue; //feat set to defined var, feat is to be set to var's value
		}
		Integer i = new Integer(featIndex);
		//if ( !var.featureIndexList.contains(i) )
		//{
			for ( int k = 0 ; k < var.featureIndexList.size() ; k++ )
			{
				int anotherFeatureIndex = ((Integer)var.featureIndexList.get(k)).intValue();
				
				//enlarge array if necessary
				while ( featureIndexesToValues.size() < anotherFeatureIndex+1 )
					featureIndexesToValues.add(null);
				
				featureIndexesToValues.set(anotherFeatureIndex,featValue);
			}
			if ( !var.featureIndexList.contains(i) )
			{
				//TODO ordered insert: bottleneck? use linked list?
				int k = 0;
				while ( k < var.featureIndexList.size() && ((Integer)var.featureIndexList.get(k)).intValue() < i.intValue() )
					k++;
				var.featureIndexList.add(k,i);
			}
			featureIndexesToValues.set(featIndex,featValue);
			List toAddTo = ((List)featureIndexesToVariables.get(featIndex));
			if ( toAddTo == null || !toAddTo.contains(var) )
			{
				if ( toAddTo == null )
				{
					toAddTo = new ArrayList();
					featureIndexesToVariables.set(featIndex,toAddTo);
				}
				toAddTo.add(var);
			}
			if ( featValue != null )
			{
				for ( int w = 0 ; w < toAddTo.size() ; w++ )
				{
					Variable v = (Variable) toAddTo.get(w);
					if ( !this.setVariableValue(v,featValue) )
						return false;
				}
			}
			
		//}		
		var.setValue(featValue);
		return true;
	}
	
	
	/*
	private static Map varMap1 = new HashMap(); //tree -> (varname->ind)
	private static Map lastIndex = new HashMap(); //tree -> last ind
	private static Map varMap2 = new HashMap(); //varname -> ind
	*/
	
	public static int treeVariableNameToIndex ( TAGTree t , String varName )
	{
		/*
		Map entry = (Map) varMap1.get(t);
		if ( entry == null )
		{
			entry = new HashMap();
			varMap1.put(t,entry);
		}
		Variable entry2 = (Variable) varName
		*/
		//System.out.println("Parsing: "+varName.substring(1));
		//(new Throwable()).printStackTrace();
		return Integer.parseInt(varName.substring(1));
	}
	
	
	
	//slow, don't use in bottlenecks
	public Variable addTreeVariable ( TAGTree t , String varName )
	{
		int index = treeVariableNameToIndex ( t , varName );
		Variable v = new Variable(t,index);
		//used by the grammar to init tree variable structures
		if ( !this.trees.contains(t) )
		{
			this.trees.add(t);
			this.variableIndexesToVariables.add(new Variable[0]);
		}
		int treeInd = this.trees.indexOf(t);
		Variable[] vars = (Variable[]) this.variableIndexesToVariables.get(treeInd);
		Variable[] newVars = new Variable[Math.max(vars.length,index+1)];
		for ( int i = 0 ; i < vars.length ; i++ )
			newVars[i] = vars[i];
		newVars[index] = v;
		this.variableIndexesToVariables.set(treeInd,newVars);
		return v;
	}
	
	public boolean linkFeatureToTreeVariable ( String featName , TAGTree t , String varName )
	{
		int index = treeVariableNameToIndex ( t , varName );
		if ( !this.trees.contains(t) )
		{
			this.trees.add(t);
			this.variableIndexesToVariables.add(new Variable[0]);
		}
		int treeInd = this.trees.indexOf(t);
		Variable[] vars = (Variable[]) this.variableIndexesToVariables.get(treeInd);
		if ( vars.length > index && vars[index] != null )
			return linkFeatureToVariable ( featureNameToIndex(featName) , vars[index] );
		else
		{
			Variable v = addTreeVariable ( t , varName );
			return linkFeatureToVariable ( featureNameToIndex(featName) , v );
		}
	}
	
	
	//true = success, false = not done (colliding values)
	/*
	public boolean linkFeatureToVariable ( String feature , String varName )
	{
		String varValue = getVariableValue ( varName );
		String featValue = getFeatureValue ( feature );
		if ( featValue != null && varValue != null && !featValue.equals(varValue) )
		{
			//feature's current value doesn't exactly match variable's current value.
			//but they may be intersecting disjunctions.
			String intersect = disjunctionIntersection ( featValue , varValue );
			if ( intersect.length() < 1 )
				return false; //values don't match
			else
			{
				varValue = intersect;
				featValue = intersect;
				//... and set the values!
			}
		}
		featureToVarNameMap.put(feature,varName);
		List featList = (List) varNameToFeatureMap.get(varName);
		if ( featList == null ) varNameToFeatureMap.put(varName,new ArrayList());
		featList = (List) varNameToFeatureMap.get(varName);
		featList.add ( feature );
		if ( featValue == null && varValue != null )
			featureToValueMap.put(feature,varValue);
		if ( featValue != null )
		{
			List feats = (List) varNameToFeatureMap.get(varName);
			if ( feats != null )
			{
				for ( int i = 0 ; i < feats.size() ; i++ )
				{
					String aLinkedFeature = (String) feats.get(i);
					featureToValueMap.put(aLinkedFeature,featValue);
				}
			}
		}
		return true;
	}
	*/
		

	
	public boolean setVariableValue ( Variable var , String value )
	{
		String currentValue = (String) getVariableValue ( var );
		if ( currentValue != null && currentValue != value ) 
		{
			String intersect = disjunctionIntersection(currentValue,value);
			if ( intersect.length() < 1 )
				return false;
			else
				value = intersect; //and set the value
		}
		for ( int i = 0 ; i < var.featureIndexList.size() ; i++ )
		{
			int featureIndex = ((Integer)var.featureIndexList.get(i)).intValue();
			featureIndexesToValues.set(featureIndex,value);
		
			while ( featureIndexesToVariables.size() < featureIndex+1 )
				featureIndexesToVariables.add(null);
			
			//update other variables' cached values
			List vars = (List) featureIndexesToVariables.get(featureIndex);
			if ( vars != null )
			{
				for ( int jh = 0 ; jh < vars.size() ; jh++ )
				{
					Variable var2 = (Variable) vars.get(jh);
					String anotherValue = (String) getVariableValue ( var2 );
					if ( anotherValue != null && anotherValue != value ) 
					{
						String intersect = disjunctionIntersection(currentValue,value);
						if ( intersect.length() < 1 )
							return false;
						else
							value = intersect; //and set the value
					}
					var2.setValue(value);
				}
			}
		}
		var.setValue(value);
		return true;
	}
	
	public boolean setFeatureValue ( String featureName , String value )
	{
		return setFeatureValue ( featureNameToIndex(featureName) , value );
	}
	
	public boolean setFeatureValue ( int featureIndex , String value )
	{
		//enlarge array if necessary
		while ( featureIndexesToValues.size() < featureIndex+1 )
			featureIndexesToValues.add(null);
		
		String currentValue = (String) featureIndexesToValues.get(featureIndex);
		if ( currentValue != null && !currentValue.equals(value) ) 
		{
			String intersect = disjunctionIntersection(currentValue,value);
			if ( intersect.length() < 1 )
				return false;
			else
				value = intersect; //and set the value
		}
		
		//enlarge array if necessary
		while ( featureIndexesToVariables.size() < featureIndex+1 )
			featureIndexesToVariables.add(null);
		
		List ofVariables = (List) featureIndexesToVariables.get(featureIndex);
		if ( ofVariables != null )
		{
			for ( int i = 0 ; i < ofVariables.size() ; i++ )
			{
				Variable v = (Variable) ofVariables.get(i);
				for ( int j = 0 ; j < v.featureIndexList.size() ; j++ )
				{
					int featureIndex2 = ((Integer)v.featureIndexList.get(j)).intValue();
					String val = getFeatureValue ( featureIndex2 );
					if ( val != null && !val.equals(value) ) 
					{
						String minval = disjunctionIntersection(val,value);
						if ( minval.length() < 1 )
							return false;
						else
							value = minval; //restrict the value to set even more!
					}
				}
			}
		}
		featureIndexesToValues.set(featureIndex,value);
		if ( ofVariables != null )
		{
			for ( int i = 0 ; i < ofVariables.size() ; i++ )
			{
				Variable v = (Variable) ofVariables.get(i);
				
				/*
				for ( int j = 0 ; j < v.featureIndexList.size() ; j++ )
				{
					int featureIndex2 = ((Integer)v.featureIndexList.get(j)).intValue();
					featureIndexesToValues.set(featureIndex2,value);
				}
				v.setValue(value);
				*/
				this.setVariableValue(v,value);
			}
		}
		
		return true;
	}
	
	
	//return true if feature didn't already have a different value
	//(i.e. if a value can be successfully set)
	/*
	public boolean setFeatureValue ( String feature , String value )
	{
		String currentValue = (String) featureToValueMap.get(feature);
		if ( currentValue != null && !currentValue.equals(value) ) 
		{
			String intersect = disjunctionIntersection(currentValue,value);
			if ( intersect.length() < 1 )
				return false;
			else
				value = intersect; //and set the value
		}
		String currentVarName = (String)featureToVarNameMap.get(feature);
		List feats = (List) varNameToFeatureMap.get(currentVarName);
		if ( feats != null )
		{
			for ( int i  = 0 ; i < feats.size() ; i++ )
			{
				String aLinkedFeature = (String) feats.get(i);
				if ( getFeatureValue(aLinkedFeature) != null && !getFeatureValue(aLinkedFeature).equals(value) && disjunctionIntersection(getFeatureValue(aLinkedFeature),value).length()<1 )
					return false;
			}
		}
		featureToValueMap.put(feature,value);
		if ( feats != null )
		{
			for ( int i  = 0 ; i < feats.size() ; i++ )
			{
				String aLinkedFeature = (String) feats.get(i);
				featureToValueMap.put(aLinkedFeature,value);
			}
		}
		return true;
	}
	*/
	
	/*
	public boolean unifies ( PlainFeatureStructure anotherFS )
	{		
		Iterator it = anotherFS.features();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();
			if ( this.isFeatureDefined(feature) && 
					this.getFeatureValue(feature) != anotherFS.getFeatureValue(feature) )
				return false;
		}
		return true;	
	}
	*/
	
	
//	public boolean unifies ( PlainFeatureStructure anotherFS )
//	{
//		if ( anotherFS == emptyInstance ) return true; //shortcut for empty fs
//		PlainFeatureStructure result = new PlainFeatureStructure(this);
//		Iterator it = anotherFS.features();
//		while ( it.hasNext() )
//		{
//			//OLD
//			/*
//			String feature = (String) it.next();
//			if ( result.isFeatureDefined(feature) && 
//					result.getFeatureValue(feature) != anotherFS.getFeatureValue(feature) &&
//					disjunctionIntersection(result.getFeatureValue(feature),anotherFS.getFeatureValue(feature)).length() < 1 )
//				return false;
//			else
//				result.setFeatureValue( feature , anotherFS.getFeatureValue(feature) ); //already sets var if present		
//			*/
//			String feature = (String) it.next();
//			if ( anotherFS.isFeatureDefined(feature) )
//			{
//				if ( !result.setFeatureValue(feature,anotherFS.getFeatureValue(feature)) )
//					return false;
//			}
//			if ( anotherFS.isFeatureLinked(feature) )
//			{
//				if ( !result.linkFeatureToVariable(feature,anotherFS.getFeatureVariable(feature)) )
//					return false;
//			}
//		}
//		return true;
//	}
//	
//	//PRE: unifies() returns true
//	//Note that unification treats feature structures as immutable objects.
//	//I.e. once all feature structures have been initted with the setter
//	//methods, they can be considered immutable to all effects.
//	public PlainFeatureStructure unify ( PlainFeatureStructure anotherFS )
//	{
//		if ( anotherFS == emptyInstance ) return this; //shortcut for empty fs
//		PlainFeatureStructure result = new PlainFeatureStructure(this);
//		Iterator it = anotherFS.features();
//		while ( it.hasNext() )
//		{
//			String feature = (String) it.next();
//			
//			//OLD
//			/*
//			if ( anotherFS.isFeatureDefined(feature) )
//				result.setFeatureValue( feature , anotherFS.getFeatureValue(feature) ); //already sets var if present
//			*/
//
//			if ( anotherFS.isFeatureDefined(feature) )
//				result.setFeatureValue(feature,anotherFS.getFeatureValue(feature));
//			if ( anotherFS.isFeatureLinked(feature) )
//				result.linkFeatureToVariable(feature,anotherFS.getFeatureVariable(feature));
//			
//		}
//		return result;
//	}
//	
//	public PlainFeatureStructure unifyKeepingVariablesOnly ( PlainFeatureStructure anotherFS )
//	{
//		if ( anotherFS == emptyInstance ) return this; //shortcut for empty fs
//		PlainFeatureStructure result = this.unify(anotherFS);
//		Iterator it = result.features();
//		while ( it.hasNext() )
//		{
//			String feature = (String) it.next();
//			String varName = result.getFeatureVariable ( feature );
//			if ( varName == null )
//				result.featureToValueMap.remove(feature);
//		}
//		return result;
//	}
//	
//	public PlainFeatureStructure unifyKeepingMyVariablesOnly ( PlainFeatureStructure anotherFS )
//	{
//		if ( anotherFS == emptyInstance ) return this; //shortcut for empty fs
//		PlainFeatureStructure result = this.unify(anotherFS);
//		Iterator it = this.features();
//		Set validFeatSet = new LinkedHashSet();
//		while ( it.hasNext() )
//		{
//			String feature = (String) it.next();
//			validFeatSet.add(feature);
//			String varName = result.getFeatureVariable ( feature );
//			if ( varName == null )
//				result.featureToValueMap.remove(feature);
//		}
//		Iterator it2 = result.features();
//		while ( it2.hasNext() )
//		{
//			String feature = (String) it2.next();
//			if ( !validFeatSet.contains(feature) )
//			{
//				result.featureToValueMap.remove(feature);
//				result.featureToVarNameMap.remove(feature);
//			}
//		}
//		return result;
//	}
//	
//	//don't unify anything, just return the result of
//	//deleting from anotherFS all variables
//	//not appearing in this FS.
//	public PlainFeatureStructure keepVariables ( PlainFeatureStructure anotherFS )
//	{
//		if ( anotherFS == emptyInstance ) return this; //shortcut for empty fs
//		PlainFeatureStructure result = new PlainFeatureStructure(anotherFS);
//		Set s = this.varNameToFeatureMap.keySet();
//		Iterator it = result.varNameToFeatureMap.keySet().iterator();
//		List l = new ArrayList();
//		while ( it.hasNext() ) { l.add(it.next()); }
//		it = l.iterator();
//		while ( it.hasNext() )
//		{
//			String varName = (String) it.next();
//			if ( !s.contains(varName) )
//			{	
//				List linkedFeats = (List) result.varNameToFeatureMap.get(varName);
//				for ( int i = 0 ; i < linkedFeats.size() ; i++ )
//				{
//					String linkedFeat = (String) linkedFeats.get(i);
//					result.featureToVarNameMap.remove(linkedFeat);
//				}
//				result.varNameToFeatureMap.remove(varName);
//			}
//		}
//		return result;
//	}
//	
//	
//	//PRE: unifies() returns true
//	/*
//	public PlainFeatureStructure unifyKeepingMyVariablesOnly ( PlainFeatureStructure anotherFS )
//	{
//		PlainFeatureStructure result = new PlainFeatureStructure(this);
//		Iterator it = anotherFS.features();
//		while ( it.hasNext() )
//		{
//			String feature = (String) it.next();
//			FSVarOrValue vov = this.getMapEntry(feature);
//			if ( vov != null && vov instanceof FSVariable )
//			{
//				FSValue newVal = new FSValue();
//				newVal.setTo(anotherFS.getValue(feature));
//				((FSVariable)vov).setValue(newVal);
//			}
//		}
//		return result;
//	}
//	*/
	
	
	
	private List getFeatureVariables ( int featureIndex )
	{
		return (List) featureIndexesToVariables.get(featureIndex);
	}
	
	/**
	 * NEW, FASTER UNIFICATION IMPLEMENTATION STARTS BELOW
	 * - Results are stored in a variable so that tentative unification can
	 * be performed in side conditions, and the result item can include the
	 * unification result without calculating unification again.
	 * - Unifications are implemented as static methods so that nulls can
	 * be easily handled.
	 */
	
	private static PlainFeatureStructure unificationResult;
	
	public static PlainFeatureStructure doUnify ( PlainFeatureStructure oneFS , PlainFeatureStructure anotherFS )
	{
		if ( oneFS == null || anotherFS == null ) return null;
		if ( anotherFS == emptyInstance ) return oneFS; //shortcut for empty fs
		if ( oneFS == emptyInstance ) return anotherFS; //shortcut for empty fs
		PlainFeatureStructure result = new PlainFeatureStructure(oneFS);
		for ( int i = 0 ; i < anotherFS.featureIndexesToValues.size() ; i ++ )
		{
			//uglier, but faster than the version below
			String val = (String) anotherFS.featureIndexesToValues.get(i);
			if ( val != null )
			{
				if ( !result.setFeatureValue(i,val) )
					return null;
			}
			/*
			if ( anotherFS.isFeatureDefined(i) )
			{
				if ( !result.setFeatureValue(i,anotherFS.getFeatureValue(i)) )
					return null;
			}*/
		}
		//create adequate pointers to variables
		for ( int i = 0 ; i < anotherFS.trees.size() ; i++ )
		{
			TAGTree t = (TAGTree) anotherFS.trees.get(i);
			if ( !result.trees.contains(t) )
			{
				result.trees.add(t);
				Variable[] anotherVars = ((Variable[])anotherFS.variableIndexesToVariables.get(i));
				Variable[] resultVars = new Variable[anotherVars.length];
				result.variableIndexesToVariables.add(resultVars);
				for ( int j = 0 ; j < anotherVars.length ; j++ )
				{
					if ( anotherVars[j] != null )
					{
						resultVars[j] = (Variable) anotherVars[j].clone();
						if ( anotherVars[j].value != null )
							if ( !result.setVariableValue(resultVars[j],anotherVars[j].value) )
								return null;
						List featsToLink = anotherVars[j].featureIndexList;
						for ( int k = 0 ; k < featsToLink.size() ; k++ )
						{
							int toLinkIndex = ((Integer)featsToLink.get(k)).intValue();
							if ( !result.linkFeatureToVariable(toLinkIndex,resultVars[j]))
							{
								return null;
							}
						}
					}
				}
				//System.out.println(result.variableIndexesToVariables.size());
				//System.out.println(((Variable[])result.variableIndexesToVariables.get(0)).length);
			}
			else //added 06.02.12 for bugfix
			{
				int ind = result.trees.indexOf(t);
				Variable[] anotherVars = ((Variable[])anotherFS.variableIndexesToVariables.get(i));
				Variable[] resultVars = ((Variable[])result.variableIndexesToVariables.get(ind));
				
				//resize resultVars if necessary
				if ( resultVars.length < anotherVars.length )
				{
					Variable[] newVars = new Variable[anotherVars.length];
					for ( int j = 0 ; j < resultVars.length ; j++ )
						newVars[j] = resultVars[j];
					result.variableIndexesToVariables.set(ind,newVars);
					resultVars = newVars;
				}
				
				for ( int j = 0 ; j < anotherVars.length ; j++ )
				{
					if ( anotherVars[j] != null )
					{

											
						
						if ( resultVars[j] == null )
						{
							//assign variable in result to features to which variable in anotherVars is assigned
							
							//resultVars[j] = (Variable) anotherVars[j].clone();
							
							resultVars[j] = result.new Variable(anotherVars[j].tree,anotherVars[j].index);
							//resultVars[j].value = anotherVars[j].value;
							if ( !result.setVariableValue(resultVars[j],anotherVars[j].value) )
								return null;
							
							List featsToLink = anotherVars[j].featureIndexList;
							for ( int k = 0 ; k < featsToLink.size() ; k++ )
							{
								int toLinkIndex = ((Integer)featsToLink.get(k)).intValue();
								if ( !result.linkFeatureToVariable(toLinkIndex,resultVars[j]))
								{
									return null;
								}
							}
						}
						else
						{
							if (anotherVars[j].value != null)
								if ( !result.setVariableValue(resultVars[j],anotherVars[j].value) )
									return null;
							
							
							//try to assign to unassigned features
							List featsToLink = anotherVars[j].featureIndexList;
							for ( int k = 0 ; k < featsToLink.size() ; k++ )
							{
								if ( resultVars[j].featureIndexList.contains(featsToLink.get(k)) )
									;
								else
								{
									int toLinkIndex = ((Integer)featsToLink.get(k)).intValue();
									if ( !result.linkFeatureToVariable(toLinkIndex,resultVars[j]))
									{
										return null;
									}
									//resultVars[j].featureIndexList.add(featsToLink.get(k));
								}
							}
						}
					}
				}
			}
		}
		return result;
	}
	
	/*
	public static PlainFeatureStructure doUnify ( PlainFeatureStructure oneFS , PlainFeatureStructure anotherFS )
	{
		if ( oneFS == null || anotherFS == null ) return null;
		if ( anotherFS == emptyInstance ) return oneFS; //shortcut for empty fs
		if ( oneFS == emptyInstance ) return anotherFS; //shortcut for empty fs
		PlainFeatureStructure result = new PlainFeatureStructure(oneFS);
		Iterator it = anotherFS.features();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();

			if ( anotherFS.isFeatureDefined(feature) )
			{
				if ( !result.setFeatureValue(feature,anotherFS.getFeatureValue(feature)) )
				{
					return null;
				}
			}
			if ( anotherFS.isFeatureLinked(feature) )
			{
				if ( !result.linkFeatureToVariable(feature,anotherFS.getFeatureVariable(feature)) )
				{
					return null;
				}
			}
		}
		return result;
	}
	*/
	
	/*
	//keep just the vars in onefs
	public static PlainFeatureStructure doKeepVariables ( PlainFeatureStructure oneFS , PlainFeatureStructure anotherFS )
	{
		if ( oneFS == null || anotherFS == null ) return null;
		if ( anotherFS == emptyInstance ) return oneFS; //shortcut for empty fs
		//if ( oneFS == emptyInstance ) return anotherFS; //shortcut for empty fs //not here, this doesn't eliminate vars!
		PlainFeatureStructure result = new PlainFeatureStructure(anotherFS);
		
		for ( int i = 0 ; i < result.trees.size() ; i++ )
		{
			TAGTree t = (TAGTree) result.trees.get(i);
			if ( (oneFS.trees.size() > 0 && !t.equals(oneFS.trees.get(0))) || (oneFS.trees.size() == 0) ) //second if condition holds on trees with no variables (remove everything!)
			{
				//remove vars from this tree
				Variable[] varsToRemove = (Variable[]) result.variableIndexesToVariables.get(i);
				for ( int j = 0 ; j < varsToRemove.length ; j++ )
				{
					Variable toRemove = varsToRemove[j];
					if ( toRemove != null )
					{
						for ( int k = 0 ; k < toRemove.featureIndexList.size() ; k++ )
						{
							int featureIndex = ((Integer)toRemove.featureIndexList.get(k)).intValue();
							((List)result.featureIndexesToVariables.get(featureIndex)).remove(toRemove);
						}
					}
				}
				result.trees.remove(i);
				result.variableIndexesToVariables.remove(i);
				i--;
			}
			
			//begin [added 06-02-15]
			else //t.equals(oneFS.trees.get(0)) holds
			{
				//remove variables with no value, which are useless
				Variable[] varsToFilter = (Variable[]) result.variableIndexesToVariables.get(i);
				for ( int j = 0 ; j < varsToFilter.length ; j++ )
				{
					Variable toFilter = varsToFilter[j];
					if ( toFilter != null )
					{
						if ( result.getVariableValue ( toFilter ) == null )
						{
							//remove the var
							for ( int k = 0 ; k < toFilter.featureIndexList.size() ; k++ )
							{
								int featureIndex = ((Integer)toFilter.featureIndexList.get(k)).intValue();
								((List)result.featureIndexesToVariables.get(featureIndex)).remove(toFilter);
							}
							varsToFilter[j] = null;
						}
					}
				}
			}
			//end [added 06-02-15]
			
		}

		
		//remove features from result which have no associated variables
		for ( int i = 0 ; i < result.featureIndexesToVariables.size() ; i++ )
		{
			List varList = (List) result.featureIndexesToVariables.get(i);
			if ( ( varList == null ) || ( varList.size() == 0 ) )
			{
				result.featureIndexesToValues.set(i,null);
			}
		}
		
		
		return result;
	}
	*/
	
	
//	keep just the vars in onefs
	public static PlainFeatureStructure doKeepVariables ( PlainFeatureStructure oneFS , PlainFeatureStructure anotherFS )
	{
		if ( oneFS == null || anotherFS == null ) return null;
		if ( anotherFS == emptyInstance ) return oneFS; //shortcut for empty fs
		//if ( oneFS == emptyInstance ) return anotherFS; //shortcut for empty fs //not here, this doesn't eliminate vars!
		PlainFeatureStructure result = new PlainFeatureStructure(anotherFS);
		
		for ( int i = 0 ; i < result.trees.size() ; i++ )
		{
			TAGTree t = (TAGTree) result.trees.get(i);
			if ( (oneFS.trees.size() > 0 && !t.equals(oneFS.trees.get(0))) || (oneFS.trees.size() == 0) ) //second if condition holds on trees with no variables (remove everything!)
			{
				//remove vars from this tree
				/*
				Variable[] varsToRemove = (Variable[]) result.variableIndexesToVariables.get(i);
				
				for ( int j = 0 ; j < varsToRemove.length ; j++ )
				{
					Variable toRemove = varsToRemove[j];
					if ( toRemove != null )
					{
						for ( int k = 0 ; k < toRemove.featureIndexList.size() ; k++ )
						{
							int featureIndex = ((Integer)toRemove.featureIndexList.get(k)).intValue();
							((List)result.featureIndexesToVariables.get(featureIndex)).remove(toRemove);
						}
					}
				}*/ //this will all be removed anyway
				
				result.trees.remove(i);
				result.variableIndexesToVariables.remove(i);
				i--;
			}
			
			//begin [added 06-02-15]
			else //t.equals(oneFS.trees.get(0)) holds
			{
				//remove variables with no value, which are useless
				Variable[] varsToFilter = (Variable[]) result.variableIndexesToVariables.get(i);
				for ( int j = 0 ; j < varsToFilter.length ; j++ )
				{
					Variable toFilter = varsToFilter[j];
					if ( toFilter != null )
					{
						if ( result.getVariableValue ( toFilter ) == null )
						{
							//remove the var
							/*
							for ( int k = 0 ; k < toFilter.featureIndexList.size() ; k++ )
							{
								int featureIndex = ((Integer)toFilter.featureIndexList.get(k)).intValue();
								((List)result.featureIndexesToVariables.get(featureIndex)).remove(toFilter);
							}
							*/
							varsToFilter[j] = null;
						}
						else
						{
							varsToFilter[j].featureIndexList = new ArrayList();
						}
					}
				}
			}
			//end [added 06-02-15]
			
		}

		
		//remove features 
		for ( int i = 0 ; i < result.featureIndexesToVariables.size() ; i++ )
		{

			result.featureIndexesToVariables.set(i,null);
			/*
			List varList = (List) result.featureIndexesToVariables.get(i);
			if ( ( varList == null ) || ( varList.size() == 0 ) )
			{
				result.featureIndexesToValues.set(i,null);
			}
			*/
		}
		for ( int i = 0 ; i < result.featureIndexesToValues.size() ; i++ )
		{
			result.featureIndexesToValues.set(i,null);
		}
		
		
		
		return result;
	}
	
	
	
	/*
	public static PlainFeatureStructure doKeepVariables ( PlainFeatureStructure oneFS , PlainFeatureStructure anotherFS )
	{
		if ( oneFS == null || anotherFS == null ) return null;
		if ( anotherFS == emptyInstance ) return oneFS; //shortcut for empty fs
		//if ( oneFS == emptyInstance ) return anotherFS; //shortcut for empty fs //not here, this doesn't eliminate vars!
		PlainFeatureStructure result = new PlainFeatureStructure(anotherFS);
		Set s = oneFS.varNameToFeatureMap.keySet();
		Iterator it = result.varNameToFeatureMap.keySet().iterator();
		List l = new ArrayList();
		while ( it.hasNext() ) { l.add(it.next()); }
		it = l.iterator();
		while ( it.hasNext() )
		{
			String varName = (String) it.next();
			if ( !s.contains(varName) )
			{	
				List linkedFeats = (List) result.varNameToFeatureMap.get(varName);
				for ( int i = 0 ; i < linkedFeats.size() ; i++ )
				{
					String linkedFeat = (String) linkedFeats.get(i);
					result.featureToVarNameMap.remove(linkedFeat);
					
					//TODO Determinar por qu aqu muchas veces llegan FS's con
					//entradas con el valor a null. Se supone que una entrada
					//nunca debera llegar a tener valor null, en todo caso
					//se quitara.
					if ( result.featureToValueMap.get(linkedFeat) == null )
						result.featureToValueMap.remove(linkedFeat);
					
				}
				result.varNameToFeatureMap.remove(varName);
			}
		}
		return result;
	}
	*/
	
	public static PlainFeatureStructure unify ( PlainFeatureStructure oneFS , PlainFeatureStructure anotherFS )
	{
		PlainFeatureStructure unifResult = doUnify ( oneFS , anotherFS );
		unificationResult = unifResult;
		return unifResult;
	}
	
	public static PlainFeatureStructure keepVariables ( PlainFeatureStructure oneFS , PlainFeatureStructure anotherFS )
	{
		PlainFeatureStructure unifResult = doKeepVariables ( oneFS , anotherFS );
		unificationResult = unifResult;
		return unifResult;
	}
	
	public static boolean unifies ( PlainFeatureStructure oneFS , PlainFeatureStructure anotherFS )
	{
		PlainFeatureStructure unifResult = doUnify ( oneFS , anotherFS );
		unificationResult = unifResult;
		return (unificationResult != null);
	}
	
	public static PlainFeatureStructure getUnificationResult ()
	{
		return unificationResult;
	}
	
	
}
