package element.fs;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;

import element.Element;
import element.ExpressionElement;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Represents an expression of the form Word-features(word) which gets
 * the given word's associated feature structure from the lexicon.
 * @author Carlos
 *
 */
public class WordFeaturesExpression extends ExpressionElement
{

		
		public WordFeaturesExpression(Element argument)
		{
			super( new Element[] { argument } );
		}
		
		public boolean isGround()
		{
			return false;
		}
		
		public Class getResultingClass()
		{
			return PlainFeatureStructure.class;
		}
		
		public String getExpressionCode(String[] memberNames) 
		{
			//TODO !!
			//This is provisional.
			
			StringBuffer sb = new StringBuffer("");
			sb.append("(element.fs.WordFeaturesExpression.getWordInformation(");
			sb.append(memberNames[0]);
			sb.append(".getName())[0]).getFeatures()");
			
			
			return new String(sb);
		}
		
		public static WordFeaturesExpression fromString ( String sparm ) throws ElementParseException
		{
			try
			{
				FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
				List expressionArgs = fep.getArguments();
				if ( expressionArgs.size() != 1 )
					throw new ElementParseException("Word Features expression taking " + expressionArgs.size() + " arguments when 1 are expected");
				Element argument = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
				return new WordFeaturesExpression ( argument );
				
			}
			catch ( FunctionExpressionParseException fepe )
			{
				throw new ElementParseException(fepe);
			}
		}

		public static void main ( String[] args ) throws Throwable
		{
			System.out.println("Ole version:");
			System.out.println("chalk:"+getWordFeaturesOldVersion("chalk"));
			System.out.println("abbess:"+getWordFeaturesOldVersion("abbess"));
			System.out.println("abbesses:"+getWordFeaturesOldVersion("abbesses"));
			System.out.println("moon:"+getWordFeaturesOldVersion("moon"));
			System.out.println("moons:"+getWordFeaturesOldVersion("moons"));
			System.out.println("moonstones:"+getWordFeaturesOldVersion("moonstones"));
			System.out.println("platypus:"+getWordFeaturesOldVersion("platypus"));
			System.out.println("halibut:"+getWordFeaturesOldVersion("halibut"));
			System.out.println("economics:"+getWordFeaturesOldVersion("economics"));
			System.out.println("world's:"+getWordFeaturesOldVersion("world's"));
			System.out.println("remember:"+getWordFeaturesOldVersion("remember"));
			System.out.println("remembered:"+getWordFeaturesOldVersion("remembered"));
			System.out.println("she:"+getWordFeaturesOldVersion("she"));
			System.out.println("that:"+getWordFeaturesOldVersion("that"));
			System.out.println("New version:");
			System.out.println("chalk:"+getWordFeatures("chalk")[0]);
			System.out.println("chalk:"+getWordFeatures("chalk")[1]);
			System.out.println("abbess:"+getWordFeatures("abbess")[0]);
			System.out.println("abbesses:"+getWordFeatures("abbesses")[0]);
			System.out.println("moon:"+getWordFeatures("moon")[0]);
			System.out.println("moons:"+getWordFeatures("moons")[0]);
			System.out.println("moonstones:"+getWordFeatures("moonstones")[0]);
			System.out.println("platypus:"+getWordFeatures("platypus")[0]);
			System.out.println("halibut:"+getWordFeatures("halibut")[0]);
			System.out.println("economics:"+getWordFeatures("economics")[0]);
			System.out.println("world's:"+getWordFeatures("world's")[0]);
			System.out.println("remember:"+getWordFeatures("remember")[0]);
			System.out.println("remembered:"+getWordFeatures("remembered")[0]);
			System.out.println("she:"+getWordFeatures("she")[0]);
			System.out.println("that:"+getWordFeatures("that")[0]);
			System.out.println("ugly:"+getWordInformation("ugly")[0].getFeatures());
			System.out.println("walks:"+getWordInformation("walks")[0].getTag()+","+getWordInformation("walks")[0].getFeatures());
			System.out.println("walks:"+getWordInformation("walks")[1].getTag()+","+getWordInformation("walks")[1].getFeatures());
			System.out.println("subdues:"+getWordInformation("subdues")[0].getTag()+","+getWordInformation("subdues")[0].getFeatures());
		}
		
		
		private static HashMap wordInformationCache = new HashMap();
		private static Set cachedWords = new LinkedHashSet();
		
		
		public static WordInformation[] getWordInformation ( String word )
		{
		
			if ( cachedWords.contains(word) )
			{
				WordInformation[] cachedData = (WordInformation[]) wordInformationCache.get(word);
				return cachedData;
			}
			else
			{
				cachedWords.add(word);
			}
			
			/*
			WordInformation[] cachedData = (WordInformation[]) wordInformationCache.get(word);
			if ( cachedData != null )
				return cachedData;
			*/
			
			try
			{
			
			
				//hardwired POS tag names, in order to tell POS tags from feature-structure
				//macros
				String[] posTags = new String[]
				{
					"A", "Ad", "Comp", "Conj", "Conj1", "Conj2",
					"D", "G", "I", "N", "N1", "NP", "P", "P1",
					"P2", "PL", "Punct", "Punct1", "Punct2", "V",
					"D1", "D2", "P3", "P4",
					
					"Det", //to D
					"Pron", //to (no idea)
					"PropN" //to (no idea)
				};
				Set posTagSet = new LinkedHashSet();
				posTagSet.addAll(Arrays.asList(posTags));
				
				FileInputStream fis1 = null;
				FileInputStream fis2 = null;
				String fileName2 = "C:\\Proyecto\\Version8\\grammars\\bigtag\\templates-modif.lex";
				fis1 = new FileInputStream(new File("C:\\Proyecto\\Version8\\grammars\\bigtag\\morph_english.flat"));
				fis2 = new FileInputStream(new File(fileName2));
	
				BufferedReader br1 = new BufferedReader ( new InputStreamReader ( fis1 ) );
				BufferedReader br2 = new BufferedReader ( new InputStreamReader ( fis2 ) );
				
				boolean foundWord = false;
				String linea;
				StringTokenizer st = null;
				
				String curPosTag;
				String curLemma;
				
				while ( (linea=br1.readLine()) != null )
				{
					st = new StringTokenizer(linea);
					if ( st.hasMoreTokens() && st.nextToken().equalsIgnoreCase(word) )
					{
						foundWord = true;
						break;
					}
				}
	
				List result = new ArrayList();
				
				
				//new word info code
				if ( foundWord )
				{
					
					System.out.println("# Word found.");
					
					//re-assemble tokens into single string
					
					String restOfLine ="";
					while ( st.hasMoreTokens() )
					{
						restOfLine += st.nextToken();
						restOfLine += " ";
					}
					restOfLine = restOfLine.trim();
					
					//at restOfLine,
					//we have something of the form lemma tag macros #lemma tag macros#lemma tag macros...
					
					StringTokenizer stAlm = new StringTokenizer(restOfLine,"#");
					while ( stAlm.hasMoreTokens() )
					{
						
						String almToken = stAlm.nextToken();
					
						//almToken is of the form lemma tag macro macro macro (...)
						
						StringTokenizer stDefinite = new StringTokenizer(almToken," ");
						
						String lemma = stDefinite.nextToken().trim();
						String tag = stDefinite.nextToken().trim();
						List macroList = new ArrayList();
						
						while ( stDefinite.hasMoreTokens() )
						{
								macroList.add ( stDefinite.nextToken().trim() );
						}
							
						//expand macros
						
						List expandedMacroList = new ArrayList();
						
						br2.close();
						fis2 = new FileInputStream(new File(fileName2));
						br2 = new BufferedReader ( new InputStreamReader ( fis2 ) );
						while ( (linea=br2.readLine()) != null )
						{
							StringTokenizer st8 = new StringTokenizer(linea);
							if ( st8.hasMoreTokens() )
							{
								String whatever = st8.nextToken();
								if ( whatever.startsWith("@") )
								{
									if ( macroList.contains(whatever.substring(1)) )
									{
										StringTokenizer st2 = new StringTokenizer(linea," \t\n\r",true);
										st2.nextToken();
										st2.nextToken();
										String rest = "";
										while ( st2.hasMoreTokens() )
											rest += st2.nextToken();
										if ( rest.endsWith("!") )
											rest = rest.substring(0,rest.length()-1);
										StringTokenizer st3 = new StringTokenizer(rest,",",false);
										while ( st3.hasMoreTokens() )
										{
											expandedMacroList.add(st3.nextToken());
										}
									}
								}
							}
						}
						
						Iterator it = expandedMacroList.iterator();
						PlainFeatureStructure pfs = new PlainFeatureStructure(true);
						
						//assemble feature structure from expanded macros
						
						while ( it.hasNext() )
						{
							String featureAssignment = (String) it.next();
							StringTokenizer st4 = new StringTokenizer(featureAssignment,"=");
							String feature = st4.nextToken().trim();
							if ( feature.startsWith("<") && feature.endsWith(">") )
								feature = feature.substring(1,feature.length()-1);
							String value = st4.nextToken().trim();
							pfs.setFeatureValue(feature,value);
							System.out.println(feature + " = " + value);
							//System.out.println("Structure: " + pfs);
							
						}
						
						WordInformation wi = new WordInformation(word,tag,lemma,pfs);
						wi.getXtagTreeFilteringInformation();
						result.add(wi);
						
						//System.out.println("Result added: " + wi.getFeatures());
						
					} //end foreach lemma-tag entry

					wordInformationCache.put(word,result.toArray(new WordInformation[0]));
					return (WordInformation[]) result.toArray(new WordInformation[0]);
					
				} //end if foundWord
				
				System.err.println("[WARNING] Couldn't get word information for: " + word);
				WordInformation wi = new WordInformation(word,word,word,PlainFeatureStructure.getEmptyInstance());
				wi.getXtagTreeFilteringInformation();
				wordInformationCache.put(word,new WordInformation[]{ wi });
				return new WordInformation[]{ wi };
				
			
			} //end try block
				
				
				/*
				if ( foundWord )
				{
					
					curLemma = st.nextToken(); //consume lemma token
					curPosTag = st.nextToken(); //consume pos tag token
					String prevPosTag = curPosTag;
					String prevLemma = curLemma;
					
					boolean goAhead = true;
					
					while ( st.hasMoreTokens() || goAhead )
					{
						
						prevPosTag = curPosTag;
					
						goAhead = false;
						
						//process next pos tag		
						
						//{Assert: curPosTag contains the POS tag we are processing}
					
						List macroList = new ArrayList();
						while ( st.hasMoreTokens() )
						{
							String entry = st.nextToken();
							if ( posTagSet.contains(entry) )
							{
								curPosTag = entry;
								goAhead = true; //process next pos tag, even if empty
								break; //following macros begin to next pos tag
							}
							if ( entry.indexOf("#") < 0 )
							{
								macroList.add ( entry.trim() );
							}
							else
							{
								StringTokenizer stAlm = new StringTokenizer ( entry , "#" );
								macroList.add( stAlm.nextToken().trim() );
							}
						}
							
						
						List expandedMacroList = new ArrayList();
						
						br2.close();
						fis2 = new FileInputStream(new File(fileName2));
						br2 = new BufferedReader ( new InputStreamReader ( fis2 ) );
						while ( (linea=br2.readLine()) != null )
						{
							StringTokenizer st8 = new StringTokenizer(linea);
							if ( st8.hasMoreTokens() )
							{
								String whatever = st8.nextToken();
								if ( whatever.startsWith("@") )
								{
									if ( macroList.contains(whatever.substring(1)) )
									{
										StringTokenizer st2 = new StringTokenizer(linea," \t\n\r",true);
										st2.nextToken();
										st2.nextToken();
										String rest = "";
										while ( st2.hasMoreTokens() )
											rest += st2.nextToken();
										if ( rest.endsWith("!") )
											rest = rest.substring(0,rest.length()-1);
										StringTokenizer st3 = new StringTokenizer(rest,",",false);
										while ( st3.hasMoreTokens() )
										{
											expandedMacroList.add(st3.nextToken());
										}
									}
								}
							}
						}
						
						Iterator it = expandedMacroList.iterator();
						PlainFeatureStructure pfs = new PlainFeatureStructure(true);
						
						while ( it.hasNext() )
						{
							String featureAssignment = (String) it.next();
							StringTokenizer st4 = new StringTokenizer(featureAssignment,"=");
							String feature = st4.nextToken().trim();
							if ( feature.startsWith("<") && feature.endsWith(">") )
								feature = feature.substring(1,feature.length()-1);
							String value = st4.nextToken().trim();
							pfs.setFeatureValue(feature,value);
							System.out.println(feature + " = " + value);
						}
						
						result.add(new WordInformation(word,prevPosTag,pfs));
					
					
					} //end while more tokens (iterating over pos tags)
					
					wordInformationCache.put(word,result.toArray(new WordInformation[0]));
					return (WordInformation[]) result.toArray(new WordInformation[0]);
					
				} //end if (foundWord)
				
				System.err.println("[WARNING] Couldn't get word information for: " + word);
				return new WordInformation[]{ new WordInformation(word,word,PlainFeatureStructure.getEmptyInstance()) };
				
			
			}
			*/
			catch ( Throwable t )
			{
				t.printStackTrace();
				return null;
			}
			
			
		}
		
		
		public static PlainFeatureStructure[] getWordFeatures ( String word )
		{
			
			try
			{
			
			
				//hardwired POS tag names, in order to tell POS tags from feature-structure
				//macros
				String[] posTags = new String[]
				{
					"A", "Ad", "Comp", "Conj", "Conj1", "Conj2",
					"D", "G", "I", "N", "N1", "NP", "P", "P1",
					"P2", "PL", "Punct", "Punct1", "Punct2", "V",
					"D1", "D2", "P3", "P4"
				};
				Set posTagSet = new LinkedHashSet();
				posTagSet.addAll(Arrays.asList(posTags));
				
				FileInputStream fis1 = null;
				FileInputStream fis2 = null;
				String fileName2 = "C:\\Proyecto\\Version8\\grammars\\bigtag\\templates-modif.lex";
				fis1 = new FileInputStream(new File("C:\\Proyecto\\Version8\\grammars\\bigtag\\morph_english.flat"));
				fis2 = new FileInputStream(new File(fileName2));
	
				BufferedReader br1 = new BufferedReader ( new InputStreamReader ( fis1 ) );
				BufferedReader br2 = new BufferedReader ( new InputStreamReader ( fis2 ) );
				
				boolean foundWord = false;
				String linea;
				StringTokenizer st = null;
				
				String curPosTag;
				
				while ( (linea=br1.readLine()) != null )
				{
					st = new StringTokenizer(linea);
					if ( st.hasMoreTokens() && st.nextToken().equalsIgnoreCase(word) )
					{
						foundWord = true;
						break;
					}
				}
	
				List featureStructures = new ArrayList();
				
				if ( foundWord )
				{
					
					st.nextToken(); //consume lemma token
					curPosTag = st.nextToken(); //consume pos tag token
					
					while ( st.hasMoreTokens() )
					{
					
						//process next pos tag		
						
						//{Assert: curPosTag contains the POS tag we are processing}
					
						List macroList = new ArrayList();
						while ( st.hasMoreTokens() )
						{
							String entry = st.nextToken();
							if ( posTagSet.contains(entry) )
							{
								curPosTag = entry;
								break; //following macros begin to next pos tag
							}
							if ( entry.indexOf("#") < 0 )
							{
								macroList.add ( entry.trim() );
							}
							else
							{
								StringTokenizer stAlm = new StringTokenizer ( entry , "#" );
								macroList.add( stAlm.nextToken().trim() );
							}
						}
							
						
						List expandedMacroList = new ArrayList();
						
						br2.close();
						fis2 = new FileInputStream(new File(fileName2));
						br2 = new BufferedReader ( new InputStreamReader ( fis2 ) );
						while ( (linea=br2.readLine()) != null )
						{
							StringTokenizer st8 = new StringTokenizer(linea);
							if ( st8.hasMoreTokens() )
							{
								String whatever = st8.nextToken();
								if ( whatever.startsWith("@") )
								{
									if ( macroList.contains(whatever.substring(1)) )
									{
										StringTokenizer st2 = new StringTokenizer(linea," \t\n\r",true);
										st2.nextToken();
										st2.nextToken();
										String rest = "";
										while ( st2.hasMoreTokens() )
											rest += st2.nextToken();
										if ( rest.endsWith("!") )
											rest = rest.substring(0,rest.length()-1);
										StringTokenizer st3 = new StringTokenizer(rest,",",false);
										while ( st3.hasMoreTokens() )
										{
											expandedMacroList.add(st3.nextToken());
										}
									}
								}
							}
						}
						
						Iterator it = expandedMacroList.iterator();
						PlainFeatureStructure pfs = new PlainFeatureStructure(true);
						
						while ( it.hasNext() )
						{
							String featureAssignment = (String) it.next();
							StringTokenizer st4 = new StringTokenizer(featureAssignment,"=");
							String feature = st4.nextToken().trim();
							String value = st4.nextToken().trim();
							pfs.setFeatureValue(feature,value);
						}
						
						featureStructures.add(pfs);
					
					
					} //end while more tokens (iterating over pos tags)
					
					
					return (PlainFeatureStructure[]) featureStructures.toArray(new PlainFeatureStructure[0]);
					
				} //end if (foundWord)
				
				return null;
			
			}
			catch ( Throwable t )
			{
				t.printStackTrace();
				return null;
			}
			
			
		}
		
		
		//test
		public static PlainFeatureStructure getWordFeaturesOldVersion ( String word ) throws Throwable
		{
			FileInputStream fis1 = new FileInputStream(new File("C:\\Proyecto\\Version8\\grammars\\bigtag\\morph_english.flat"));
			FileInputStream fis2 = new FileInputStream(new File("C:\\Proyecto\\Version8\\grammars\\bigtag\\templates-modif.lex"));
			BufferedReader br1 = new BufferedReader ( new InputStreamReader ( fis1 ) );
			BufferedReader br2 = new BufferedReader ( new InputStreamReader ( fis2 ) );
			
			boolean foundWord = false;
			String linea;
			StringTokenizer st = null;
			
			while ( (linea=br1.readLine()) != null )
			{
				st = new StringTokenizer(linea);
				if ( st.hasMoreTokens() && st.nextToken().equalsIgnoreCase(word) )
				{
					foundWord = true;
					break;
				}
			}

			if ( foundWord )
			{
				st.nextToken(); //consume lemma token
				List macroList = new ArrayList();
				while ( st.hasMoreTokens() )
				{
					String entry = st.nextToken();
					if ( entry.indexOf("#") >= 0 )
						macroList.add ( entry.trim() );
					else
					{
						StringTokenizer stAlm = new StringTokenizer ( entry , "#" );
						macroList.add( stAlm.nextToken().trim() );
					}
				}
					
				
				List expandedMacroList = new ArrayList();
				
				while ( (linea=br2.readLine()) != null )
				{
					st = new StringTokenizer(linea);
					if ( st.hasMoreTokens() )
					{
						String whatever = st.nextToken();
						if ( whatever.startsWith("@") )
						{
							if ( macroList.contains(whatever.substring(1)) )
							{
								StringTokenizer st2 = new StringTokenizer(linea," \t\n\r",true);
								st2.nextToken();
								st2.nextToken();
								String rest = "";
								while ( st2.hasMoreTokens() )
									rest += st2.nextToken();
								if ( rest.endsWith("!") )
									rest = rest.substring(0,rest.length()-1);
								StringTokenizer st3 = new StringTokenizer(rest,",",false);
								while ( st3.hasMoreTokens() )
								{
									expandedMacroList.add(st3.nextToken());
								}
							}
						}
					}
				}
				
				Iterator it = expandedMacroList.iterator();
				PlainFeatureStructure pfs = new PlainFeatureStructure(true);
				
				while ( it.hasNext() )
				{
					String featureAssignment = (String) it.next();
					StringTokenizer st4 = new StringTokenizer(featureAssignment,"=");
					String feature = st4.nextToken().trim();
					String value = st4.nextToken().trim();
					pfs.setFeatureValue(feature,value);
				}
				
				return pfs;
				
			}
			
			return null;
			
			
		}
		


}
