/*
 * Created at regulus on 26-nov-2004 19:47:45
 * as file Term.java on package element.unification
 */
package element.unification;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import java.util.Map;
import java.util.StringTokenizer;

import element.Element;
import element.SimpleElement;

/**
 * @author carlos
 *
 * Created at regulus, 26-nov-2004 19:47:45
 */
public class Term extends SimpleElement
{

	/**
	 * Trminos hijos de este trmino lgico. Un trmino tendr hijos
	 * cuando sea un funtor, del estilo de f(x,Y,f(Z),...)
	 * */
	private Term[] children;
	
	/**
	 * Si este trmino lgico se considera ground o no desde el punto de vista
	 * del esquema de anlisis sintctico.
	 */
	private boolean ground;
	
	/**
	 * Si este trmino lgico representa una variable.
	 */
	private boolean isVariable;
	
	/**
	 * Elemento de anlisis asociado al trmino lgico, si es una constante.
	 * Esto puede ser un smbolo, etc.
	 */
	private Element associatedElement;
	
	/**
	 * Nombre del trmino lgico.
	 */
	private String name;
	
	/**
	 * Variable temporal utilizada en la unificacin.
	 */
	private LinkedList unificationStackMember1 = new LinkedList();
	/**
	 * Variable temporal utilizada en la unificacin.
	 */
	private LinkedList unificationStackMember2 = new LinkedList();
	
	/**
	 * Igualdad de trminos lgicos.
	 */
	public boolean equals ( Object o )
	{
		if ( !(o instanceof Term ) ) return false;
		Term t = (Term) o;
		if ( associatedElement == null )
			if ( t.associatedElement != null ) return false;
		if ( associatedElement != null && !associatedElement.equals(t.associatedElement) ) return false;
		if ( ground != t.ground ) return false;
		if ( children == null )
			if ( t.children != null ) return false;
		if ( children != null && children.length != t.children.length ) return false;
		if ( !name.equals(t.name) ) return false;
		if ( children != null )
		{
			for ( int i = 0 ; i < children.length ; i++ )
			{
				if ( !children[i].equals(t.children[i]) ) return false;
			}
		}
		return true;
	}
	
	/**
	 * Fija el elemento asociado a este trmino lgico.
	 */
	void setAssociatedElement ( Element elt )
	{
		associatedElement = elt;
	}
	
	/**
	 * Obtiene el elemento asociado a este trmino.
	 */
	Element getAssociatedElement (  )
	{
		return associatedElement;
	}
	
	/**
	 * Fija los trminos hijos de ste.
	 */
	void setChildren ( Term[] children )
	{
		this.children=children;
	}
	
	/**
	 * Prueba de unidad sencilla para trminos lgicos.
	 */
	public static void main ( String[] args )
	{
		Term root = new Term(true,"X");
		root.isVariable = true;
		Term root2 = new Term(true,"f");
		root2.isVariable=false;
		root2.children = new Term[]{root};
		System.out.println(root.unify(root2));
		
		Term root3 = new Term(true,"k");
		root3.isVariable=false;
		System.out.println(root2.unify(root3));
		System.out.println(root.unify(root3));
		
		Term root4 = new Term(true,"f");
		root4.isVariable=false;
		root4.children=new Term[]{root3};
		System.out.println(root2.unify(root4));
		System.out.println(root2.unification(root4));
	}
	
	public int numTermChildren()
	{
		if ( children == null ) return 0;
		else return children.length;
	}
	
	public Term getTermChild(int i)
	{
		return children[i];
	}
	
	/**
	 * Devuelve el resultado de unificar este trmino lgico con el trmino lgico
	 * dado. Ntese que este mtodo puede modificar los trminos de entrada, que
	 * por lo tanto deberan ser clonados en caso de que se quieran seguir usando
	 * para algo en su forma original.
	 * @param anotherTerm Trmino con el que unificamos.
	 * @return Un trmino lgico resultado de aplicar el unificador ms general
	 * de ambos trminos, si existen, o bien null si no se pueden unificar.
	 */
	public Term unification ( Term anotherTerm )
	{
		Unifier u = this.unify ( anotherTerm );
		if ( u != null )
			return u.apply(this);
		else
			return null;
	}
	
	/**
	 * Devuelve el conjunto de sustituciones (Unifier) ms general que hace que este trmino
	 * lgico unifique con el trmino lgico dado como parmetro, si existe. 
	 * De lo contrario, devuelve null.
	 * Ntese que este mtodo no modifica ninguno de los dos trminos lgicos,
	 * al contrario que unification().
	 * @param anotherTerm Trmino con el que unificamos.
	 * @return Un unificador tal que u.apply(this) y u.apply(anotherTerm) dan trminos
	 * lgicos iguales segn equals.
	 */
	public Unifier unify ( Term anotherTerm )
	{
		unificationStackMember1.clear();
		unificationStackMember2.clear();
		Unifier u = new Unifier();
		unificationStackMember1.add(this);
		unificationStackMember2.add(anotherTerm);
		while ( !unificationStackMember1.isEmpty() )
		{
			Term x = (Term) unificationStackMember1.removeLast();
			Term y = (Term) unificationStackMember2.removeLast();
			if ( x.isVariable )
			{
				Iterator it = unificationStackMember1.iterator();
				while ( it.hasNext() )
				{
					((Term)it.next()).applySubstitution(x,y);
					
				}
				Iterator it2 = unificationStackMember2.iterator();
				while ( it2.hasNext() )
				{
					((Term)it2.next()).applySubstitution(x,y);
				}
				u.applySubstitution(x,y);
				u.add(x,y);
			}
			if ( y.isVariable )
			{
				Iterator it = unificationStackMember1.iterator();
				while ( it.hasNext() )
				{
					((Term)it.next()).applySubstitution(y,x);	
				}
				Iterator it2 = unificationStackMember2.iterator();
				while ( it2.hasNext() )
				{
					((Term)it2.next()).applySubstitution(y,x);
				}
				u.applySubstitution(y,x);
				u.add(y,x);
			}
			if ( !y.isVariable && !x.isVariable )
			{
				if ( y.numTermChildren() != x.numTermChildren() )
					return null;
				else if ( y.numTermChildren() == 0 )
				{
					if ( x.equals(y) ) continue;
					else return null;
				}
				else
				{
					for ( int i = 0 ; i < x.numTermChildren() ; i++ )
					{
						unificationStackMember1.add(x.children[i]);
						unificationStackMember2.add(y.children[i]);
					}
				}
			}
			
		} //while
		
		return u;
	}
	
	public void applySubstitution ( Term t1 , Term t2 )
	{
		if ( children != null )
		for ( int i = 0 ; i < children.length ; i++ )
		{
			if ( children[i] == t1 )
				children[i] = t2;
			children[i].applySubstitution(t1,t2);
		}
	}
	
	/**
	 * Elemento de un "parsing schema" que representa un trmino lgico. El trmino
	 * lgico puede ser una variable, una constante o un rbol; aunque se trate como
	 * un elemento simple (SimpleElement).
	 */
	public Term( boolean ground , String name ) 
	{
		this.ground = ground;
		this.name = name;
	}
	
	/**
	 * Crea un trmino lgico ground asociado al elemento dado.
	 * @param associatedElement
	 */
	public Term ( Element associatedElement )
	{
		this.associatedElement = associatedElement;
		this.name = associatedElement.toString();
		this.isVariable = false;
		this.ground = true;
	}
	
	/**
	 * Crea un trmino lgico asociado al elemento dado y con los hijos dados.
	 */
	public Term ( Element associatedElement , Term[] children )
	{
		this(associatedElement);
		this.children=children;
	}
	
	/**
	 * Proporciona una representacin en forma de cadena del trmino lgico.
	 */
	public String toString()
	{
		if  ( children == null || children.length == 0 ) return name;
		StringBuffer sb = new StringBuffer(name);
		sb.append("(");
		for ( int i = 0 ; i < children.length ; i++ )
		{
			if ( i>0 ) sb.append(", ");
			if ( children[i] != null )
			sb.append(children[i].toString());
		}
		sb.append(")");
		return new String(sb);
	}
	
	public boolean isGround()
	{
		return ground;
	}
	
	/**
	 * Nos dice si este trmino empareja con el trmino dado.
	 */
	public boolean matches ( Element e )
	{
		if ( !(e instanceof Term) ) return false;
		if ( !this.isGround() ) return true;
		else return this.unify((Term)e) != null;
	}

	public static Term groundFromString ( String s )
	{
		//ignore first two characters, %%
		if ( s.length() < 2 )
			return null;
		return groundFromCleanString(s.substring(2));
	}
	
	/**
	 * Devuelve el cdigo Java para que los trminos ground aparezcan en expresiones.
	 */
	public String getExpressionCode()
	{
		if ( associatedElement != null && associatedElement instanceof SimpleElement )
		{
			if ( children == null || children.length == 0 )
				return "new element.unification.Term( " + ((SimpleElement)associatedElement).getExpressionCode() + ")";
			else
			{
				StringBuffer sb = new StringBuffer();
				
				sb.append("new element.unification.Term( ");
				sb.append(((SimpleElement)associatedElement).getExpressionCode());
				sb.append(", ");

				StringBuffer childrenArray = new StringBuffer("new element.unification.Term[] { ");
				for ( int i = 0 ; i < children.length ; i++ )
				{
					
					if ( i > 0 ) childrenArray.append(", ");

					childrenArray.append(children[i].getExpressionCode());

				}		
				childrenArray.append("}");
				sb.append(childrenArray);
				sb.append(")");
				return new String(sb);
			}
		}
			else if ( ground )
	        return "element.unification.Term.groundFromCleanString(\""+name+"\")";
	    else
	        return null;
	}
	
	

	
	//string que no incluya el %%.
	public static Term groundFromCleanString ( String s )
	{
		
		StringTokenizer st = new StringTokenizer ( s , "();" , true );
		String base = st.nextToken();
		Term t = new Term ( true , s );
		List children = new ArrayList();
		while ( st.hasMoreTokens() )
		{
			String tok = st.nextToken();
			if ( tok.equals(")") )
			{
				//Syntax Error.
			}
			else if ( tok.equals("(") )
			{
				String anotherTok = "";
				while ( !anotherTok.equals(")") )
				{
					if ( !st.hasMoreTokens() )
					{
						//Syntax Error.
						break;
					}
					anotherTok = st.nextToken();
					if ( anotherTok.equals(",") || anotherTok.equals(")") );
					else
					{
						Term child = groundFromCleanString(anotherTok);
						children.add(child);
					}
				}
			}
		}
		t.children = (Term[])children.toArray(new Term[0]);
		return t;
	}
	
	private static Map nonGroundTermsMap = new HashMap();
	
	public static Term nonGroundFromString ( String s )
	{
		//ignore first character
		if ( s.length() < 1 ) return null;
	    if ( nonGroundTermsMap.get(s) != null )
	        return (Term) nonGroundTermsMap.get(s);
	    else
	    {
	        //nuevo
	        Term nuevo = new Term(false,s);
	        nonGroundTermsMap.put(s,nuevo);
	        return nuevo;
	    }
	}
	
	
}
