/*
 * Created at regulus on 27-nov-2004 16:54:55
 * as file TermConstructorExpression.java on package element.unification
 */
package element.unification;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;

import element.Element;
import element.ExpressionElement;
import element.SimpleElement;

/**
 * @author carlos
 *
 * Created at regulus, 27-nov-2004 16:54:55
 */
public class TermConstructorExpression extends ExpressionElement 
{
	
	//A ( B , C )
	//es una expresin con hijos A,B,C.
	//(el primer hijo es la raz)
	//Los hijos no tienen por qu ser Terms.


	public TermConstructorExpression(Element[] children) 
	{
		super(children);
	}


	public Class getResultingClass() 
	{
		return Term.class;
	}


	public boolean isGround() 
	{
		return false;
	}

	public boolean matches(Element e) 
	{
		return false;
	}
	

	public static List parseChildren ( StringTokenizer st ) throws ElementParseException
	{
		ArrayList children = new ArrayList();
		boolean acceptSemicolon = false;
		for ( ;; )
		{
			
			String tok = st.nextToken();
			
			if ( tok.equals(")") )
			{
				return children;
			}
			if ( tok.equals(";") && acceptSemicolon )
			{
				acceptSemicolon = false;
				continue;
			}
			else if ( tok.equals(";") || tok.equals("(") )
				throw new ElementParseException("Unexpected symbol " + tok + " when parsing TermConstructorExpression");			
			else
			{

				String base = tok;
				Element root = ElementParser.getCurrentInstance().parse(base);
				List subchildren = new ArrayList();
				//children.add(root);
				if ( st.hasMoreTokens() )
				{
					String anotherTok = st.nextToken();
					if ( anotherTok.equals("(") )
					{
						subchildren.add(root);
						subchildren.addAll(parseChildren(st));
						acceptSemicolon = true;
						TermConstructorExpression t = new TermConstructorExpression((Element[])subchildren.toArray(new Element[0]));
						children.add(t);
						continue;
					}
					else if ( anotherTok.equals(";") )
					{
						subchildren.add(root);
						TermConstructorExpression t = new TermConstructorExpression((Element[])subchildren.toArray(new Element[0]));
						children.add(t);
						continue;
					}
					else if ( anotherTok.equals(")"))
					{
						subchildren.add(root);
						TermConstructorExpression t = new TermConstructorExpression((Element[])subchildren.toArray(new Element[0]));
						children.add(t);
						return children;
					}
					else 
					{
						throw new ElementParseException("Unexpected symbol " + anotherTok + " when parsing TermConstructorExpression");
					}
				}	
			}
		}
		
		
		
	}
	
	public static TermConstructorExpression fromString2 ( String s ) throws ElementParseException
	{
		
		if ( s.length() < 2 ) throw new ElementParseException("TermConstructorExpression string " + s + " hasn't enough length");
		s = s.substring(2);
		
		StringTokenizer st = new StringTokenizer ( s , "();" , true );
		String base = st.nextToken();
		Element root = ElementParser.getCurrentInstance().parse(base);
		//Term rootChild;
		//if ( root instanceof Term ) rootChild = (Term)root;
		//else rootChild = new Term(root);
		Element rootChild = root;
		
		//List children = new ArrayList();
		
		List children = new ArrayList();
		children.add(rootChild);
		if ( st.hasMoreTokens() )
		{
			if ( st.nextToken().equals("(") )
			{
				children.addAll(parseChildren(st));
			}
			else
			{
				throw new ElementParseException("Unexpected symbol when parsing TermConstructorExpression");
			}
		}		

		TermConstructorExpression t = new TermConstructorExpression((Element[])children.toArray(new Element[0]));
		//t.setAssociatedElement(rootChild);
		return t;
	}
	
	//para interpretar una expresin de tipo "add tree", por ejemplo
	//%%addtree( %%S(%%NP;%%VP) ; %%RP(%%SP) )
	//dara lugar a
	//%%S(%%NP;%%VP;%%RP(%%SP)).
	//Es decir, la semntica es que el segundo rbol se aade como hijo a la raz
	//del primero.
	public static TermConstructorExpression addTreeExpressionFromString ( String s ) throws ElementParseException
	{
		
		if ( s.length() < 2 ) throw new ElementParseException("TermConstructorExpression string " + s + " hasn't enough length");
		s = s.substring(2);
		
		StringTokenizer st = new StringTokenizer ( s , "();" , true );
		
		
		List arguments = new ArrayList();
		
		while ( st.hasMoreTokens() )
		{
			String tok = st.nextToken();
			if ( tok.equals(")") )
			{
				throw new ElementParseException ( "Syntax error in term constructor expression " + s);
			}
			else if ( tok.equals("(") )
			{
				String anotherTok = "";
				String cumulative = "";
				while ( !anotherTok.equals(")") )
				{
					if ( !st.hasMoreTokens() )
					{
						//Syntax Error.
						break;
					}
					anotherTok = st.nextToken();
					if ( anotherTok.equals(";") || anotherTok.equals(")") ) ;
					else
					{						
						Element child = ElementParser.getCurrentInstance().parse(anotherTok);
						if ( child instanceof Term )
							arguments.add(child);
						else if ( child instanceof TermConstructorExpression )
							arguments.add(new Term(((TermConstructorExpression)child).getChild(0)));
						else
							arguments.add(new Term(child));
					}
				}
			}
		} //while (get arguments)
		
		if ( arguments.size() < 2 )
		{
			throw new ElementParseException ( "Syntax error in term constructor expression " + s);
		}
		
		ArrayList children = new ArrayList();
		Term rootTree = (Term) arguments.get(0);
		//children.add(rootTree.getAssociatedElement());
		for ( int i = 0 ; i < rootTree.numTermChildren() ; i++ )
			children.add(rootTree.getTermChild(i));
		children.add ( (Term) arguments.get(1) );
		
		TermConstructorExpression t = new TermConstructorExpression((Element[])children.toArray(new Element[0]));
		return t;	
		
	}
	
	public static TermConstructorExpression fromString ( String s ) throws ElementParseException
	{
		
		if ( s.length() < 2 ) throw new ElementParseException("TermConstructorExpression string " + s + " hasn't enough length");
		s = s.substring(2);
		
		StringTokenizer st = new StringTokenizer ( s , "();" , true );
		String base = st.nextToken();
		Element root = ElementParser.getCurrentInstance().parse(base);
		Element rootChild = root;
		List children = new ArrayList();
		children.add(rootChild);
		while ( st.hasMoreTokens() )
		{
			String tok = st.nextToken();
			if ( tok.equals(")") )
			{
				throw new ElementParseException ( "Syntax error in term constructor expression " + s);
			}
			else if ( tok.equals("(") )
			{
				String anotherTok = "";
				String cumulative = "";
				while ( !anotherTok.equals(")") )
				{
					if ( !st.hasMoreTokens() )
					{
						//Syntax Error.
						break;
					}
					anotherTok = st.nextToken();
					if ( anotherTok.equals(";") || anotherTok.equals(")") ) ;
					else
					{
						Element child = ElementParser.getCurrentInstance().parse(anotherTok);
						
						
						if ( child instanceof Term )
							children.add(child);
						else if ( child instanceof TermConstructorExpression )
							children.add(child);
						else
							children.add(new TermConstructorExpression(new Element[]{child}));
					}
				}
			}
		}
		TermConstructorExpression t = new TermConstructorExpression((Element[])children.toArray(new Element[0]));
		return t;
	}
	
	public String getExpressionCode( String[] memberNames  )
	{
		
	
		Element associatedElement = children[0];
		if ( associatedElement != null && associatedElement instanceof SimpleElement )
		{
			if ( children.length == 1 )
				return "new element.unification.Term( " + memberNames[0] + ")";
			else
			{
				StringBuffer sb = new StringBuffer();
				
				sb.append("new element.unification.Term( ");
				sb.append(memberNames[0]);
				sb.append(", ");

				StringBuffer childrenArray = new StringBuffer("new element.unification.Term[] { ");
				for ( int i = 1 ; i < children.length ; i++ )
				{
					
					if ( i > 1 ) childrenArray.append(", ");

					childrenArray.append(memberNames[i]);

				}		
				childrenArray.append("}");
				sb.append(childrenArray);
				sb.append(")");
				return new String(sb);
			}
		}
	    else
	    {
	    	System.err.println("[ERROR] Root of tree expression must be a SimpleElement.");
	        return null; 
	    }
		
	}
	
	
	
}
