/*
 * Created at denebola on 24-ago-2004 11:33:30
 *
 */
package generator;

import element.CompositeElement;
import element.SequenceElement;
import element.SimpleElement;
import element.ExpressionElement;
import java.util.*;


/**
 * <p>Visitante que genera cdigo para expresiones.</p>
 * <p>Created at denebola, 24-ago-2004 11:33:30</p>
 * @author carlos
 */
public class ExpressionVisitor extends CodeGeneratingVisitor
{

    /**
     * @param toAppendTo Buffer en el que se agregar el cdigo de la expresin.
     * @param cg Generador de cdigo que incluye informacin de atributos.
     */
    public ExpressionVisitor (  StringBuffer toAppendTo, DeductiveStepCodeGenerator cg )
    {
        super ( null, toAppendTo, cg );
    }

    /**
     * Genera la parte de una expresin correspondiente a un elemento simple,
     * siempre que implemente el interfaz ExpressionMember.
     */
    public void visitSimpleElement ( SimpleElement se )
    {
        if ( se.isGround() )
        {
            //elemento que ya apareca ground en la expresin (p.ej. el 1 en i+1)
            toAppendTo.append(se.getExpressionCode());
        }
        else if ( generator.isGround(generator.attributeFor(se)) )
        {
            //elemento que aparece no-ground en la expresin; pero que ahora tiene
            //algn valor concreto
            //(p.ej. la segunda i en [A,i,i+1] al matchear con [A,3,4]: cuando llegamos
            //a i+1 ya sabemos que i vale 3)
        	//System.err.println("Attribute for "+se+": " +generator.attributeFor(se).getName() );
            toAppendTo.append(generator.attributeFor(se).getName());
        }
        else
        {
            //elemento que aparece no-ground en la expresin y del que adems no
            //conocemos su valor
            System.out.println("[ERROR]: " + se + " can't be part of an expression");
            (new Throwable()).printStackTrace();
        }
    }

	/**
	 * Genera la parte de una expresin correspondiente a un elemento compuesto,
	 * siempre que implemente el interfaz ExpressionMember.
	 */
    public void visitCompositeElement ( CompositeElement ce )
    {
        //los composite no pueden tomar parte en expresiones.
        System.err.println("[ERROR]: " + ce + " is a CompositeElement and can't be part of an expression");
    }
    
    public void visitSequenceElement ( SequenceElement zome )
    {
    	System.err.println("[ERROR]: " + zome + " is a SequenceElement and can't be part of an expression");
    }
    
    /**
     * Genera la expresin correspondiente a un elemento de expresin.
     * @param ee
     */
    public void visitExpressionElement ( ExpressionElement ee )
    {
        ArrayList memberStrings = new ArrayList();
        for ( int i = 0 ; i < ee.numChildren() ; i++ )
        {
            StringBuffer sb = new StringBuffer();
            StringBuffer oldBuffer = this.toAppendTo;
            this.toAppendTo = sb;
            ee.getChild(i).accept(this);
            this.toAppendTo = oldBuffer;
            memberStrings.add(sb.toString());
        }
        toAppendTo.append(ee.getExpressionCode((String[])memberStrings.toArray(new String[0]),generator));
    }

}
