/*
 * Created at regulus on 28-oct-2004 19:29:12
 * as file SearchDescriptorNode.java on package generator
 */
package generator;

/**
 * <p>Cada instancia de esta clase es un nodo de un "descriptor de bsqueda". El descriptor de bsqueda
 * es un rbol que se generar para cada peticin a la funcin de recuperar items a travs
 * de los ndices, en la clase ItemHandler.</p>
 * <p>No se debe confundir el descriptor de bsqueda con la peticin de bsqueda en s. La peticin de bsqueda es
 * lo que el cdigo generado crea para pasarlo al ItemHandler y que ste le devuelva un tem.
 * La peticin de bsqueda es un rbol formado por arrays de objetos, donde puede haber elementos simples
 * ground, elementos simples nulos y elementos tipo secuencia no ground. El
 * descriptor de bsqueda es una estructura que slo maneja el generador de cdigo, sin que
 * trascienda al cdigo generado. La estructura la forman los SearchDescriptorNodes, y cada nodo
 * tiene asociado un objeto Class representando la clase del objeto que se est pidiendo,
 * para los elementos que sean ground en la peticin, o bien un null para los nulos en 
 * la peticin (no nos interesa saber la clase de los objetos que no se piden en la peticin).</p>
 * <p>Por ejemplo, cuando en CYK buscamos un nodo de la forma [C,j,?]; el cdigo generado
 * va a generar una peticin que va a ser un Object[]{C,j,null}. El descriptor de bsqueda
 * correspondiente ser SearchDescriptorNode[SearchDescriptorNode[Symbol],SearchDescriptorNode[IntElement],
 * SearchDescriptorNode[null]] y su funcin no es otra que ayudar a generar la clase ItemHandler
 * para que pueda responder a peticiones del tipo de la anterior.</p>
 * <p>Created 12-ago-2004 11:35:16</p>
 * @author carlos
 */
public class SearchDescriptorNode
{
    
	/**
	 * Nodos hijos.
	 */
    private SearchDescriptorNode[] children;
    
    /**
     * La etiqueta del nodo es una clase.
     */
    private Class clase;
    
    /**
     * Compara dos SearchDescriptorNode. Dos SearchDescriptorNode son iguales si tienen asociada
     * la misma clase, tienen el mismo nmero de subrboles y dichos subrboles son
     * iguales.
     */
    public boolean equals ( Object o )
    {
    	if ( !(o instanceof SearchDescriptorNode) ) return false;
    	SearchDescriptorNode stn = (SearchDescriptorNode) o;
    	if ( this.clase != stn.clase ) return false;
    	if ( this.children == null )
    		return (stn.children==null);
    	else if ( stn.children == null ) return false;
    	if ( this.children.length != stn.children.length ) return false;
    	for ( int i = 0 ; i < children.length ; i++ )
    	{
    		if ( !this.children[i].equals(stn.children[i]) )
    			return false;
    	}
    	return true;
    }
    
    /**
     * Devuelve el nmero de nodos hijos que tiene este nodo en el rbol descriptor de bsqueda.
     * @return Nmero de nodos hijos de este nodo en el rbol descriptor de bsqueda.
     */
    public int numChildren()
    {
        return children.length;
    }
    
    /**
     * Devuelve el i-simo nodo hijo de este nodo en el rbol descriptor de bsqueda.
     * @param i ndice del nodo hijo que queremos obtener.
     * @return I-simo nodo hijo de ste en el arbol descriptor de bsqueda.
     */
    public SearchDescriptorNode getChild ( int i )
    {
        return children[i];
    }
    
    /**
     * Devuelve el nodo del descriptor de bsqueda en una direccin dada como lista de Integer.
     * @param address Direccin de un nodo del descriptor de bsqueda, dada como lista de Integer.
     * @return Nodo que ocupa la direccin dada en el rbol descriptor de bsqueda.
     */
    public SearchDescriptorNode nodeAtAddress ( java.util.List address )
    {
        SearchDescriptorNode currentNode = this;
        for ( int i = 0 ; i < address.size() ; i++ )
        {
            currentNode = currentNode.getChild(((Integer)address.get(i)).intValue());
        }
        return currentNode;
    }
    
    //nodo compuesto
    /**
     * Construye un nodo no hoja cuyos hijos, por orden, son los elementos
     * del array dado.
     * @param children Array que contiene los nodos hijos del nodo que
     * queremos construir.
     */
    public SearchDescriptorNode ( SearchDescriptorNode[] children )
    {
        this.children = children;
        this.clase = Object[].class;
    }
    
    //nodo simple
    /**
     * Construye un nodo de rbol descriptor de bsqueda, sin hijos, y asociado
     * a la clase dada (que puede ser null).
     */
    public SearchDescriptorNode ( Class clase )
    {
        this.clase = clase;
    }
    
    /**
     * Obtiene la clase asociada a este nodo del rbol descriptor de bsqueda.
     * @return Clase de elementos (subclase de Element) asociada a este
     * nodo de rbol descriptor de bsqueda.
     */
    public Class getAssociatedClass()
    {
        return clase;
    }
    
    /**
     * Cambia la clase de elemento asociada a este nodo del rbol descriptor de bsqueda.
     * @param clase Nueva clase, subclase de element.Element, que queremos
     * que represente este nodo del rbol descriptor de bsqueda.
     */
    public void setAssociatedClass( Class clase )
    {
    	this.clase = clase;
    }
    
    //nodo nulo
    /**
     * Construye un nodo de rbol descriptor de bsqueda sin hijos y con clase null
     * (es decir, representando algn elemento no-ground)
     */
    public SearchDescriptorNode ( )
    {
        this.clase = null;
    }
    
    /**
     * Mtodo auxiliar de toString().
     * @param sb Buffer donde se escribir la descripcin de este descriptor de bsqueda en forma de cadena.
     */
    private void dumpString(StringBuffer sb)
    {
    	if ( children != null && numChildren() > 0 )
    	{
    		sb.append("[");
    		for ( int i = 0 ; i < numChildren() ; i++ )
    		{
    			getChild(i).dumpString(sb);
    			if ( i < numChildren() )
    				sb.append(",");
    		}
    		sb.append("]");
    	}
    	else
    	{
    		if ( getAssociatedClass() == null )
    			sb.append("?");
		    else
		    	sb.append(getAssociatedClass().getName());
    	}
    }
    
    /**
     * Devuelve una representacin en forma de cadena de este descriptor de bsqueda.
     */
    public String toString()
    {
    	StringBuffer sb = new StringBuffer();
    	sb.append("Descriptor de Bsqueda ");
    	dumpString(sb);
    	return sb.toString();
    }
    
}
