package grammar.cfg.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * This class converts a context-free grammar file with a format such as this
 * 
 * S
 * NP VP
 * NP VP PP
 * 
 * to one with a format such as this:
 * 
 * S -> NP VP
 * S -> NP VP PP
 * 
 * @author Carlos Gmez
 * Created at mortadelo, 28/11/2006 12:18:07
 *
 */
public class CompactToStandardCFGConverter 
{

	public static String IN_FILE = "C:\\Proyecto\\Version9\\grammars\\ct-grammar-eval.txt";
	public static String OUT_FILE = "C:\\Proyecto\\Version9\\grammars\\ct-grammar-eval-converted.txt";
	
	
	public static void main ( String[] args ) throws FileNotFoundException,IOException
	{
	
		convert ( new File(IN_FILE) , new File(OUT_FILE) );
		
		
	}
	
	public static void convert ( File input , File output ) throws FileNotFoundException,IOException
	{
		
		BufferedReader br = new BufferedReader ( new InputStreamReader ( new FileInputStream ( input ) ) );
		PrintWriter pw = new PrintWriter ( new OutputStreamWriter ( new FileOutputStream ( output ) ) );
		
		String currentLHS = null; //current rule left-hand-side
		String currentLine = " "; //last line read
		boolean nextLineIsLHS = true;
		
		StringBuffer outBuf = new StringBuffer("");
		
		while ( currentLine != null )
		{
			if ( currentLine.startsWith(";") ) ;
			else if ( currentLine.trim().equals("") )
			{
				nextLineIsLHS = true;
			}
			else
			{
				if ( nextLineIsLHS )
				{
					//read left hand side of rule
					currentLHS = currentLine.trim();
					nextLineIsLHS = false;
				}
				else
				{
					//read right hand side
					StringTokenizer st = new StringTokenizer ( currentLine.trim() );
					
					outBuf.append(currentLHS);
					outBuf.append(" ->");
					while ( st.hasMoreTokens() ) 
					{
						outBuf.append(" ");
						outBuf.append(st.nextToken().trim());
					}
					outBuf.append("\n");
				}
			}
			
			currentLine = br.readLine();
		}
		
		pw.print(outBuf);
		pw.close();
		
	}
	
	
}
