package parser.sparser.tree;
/**
 * Nodo que representa un paso deductivo.
 */
public class DeductiveStepNode
{

	private String name;
	private AntecedentNode antecedent; //has inside a list of items and forall exprs.
	private AntecedentNode negations; //negated items
	private ParseTreeNode consequent; //an ItemNode or a ForallScopeNode
	private ParseTreeNode conditions; //instantiation side conditions. Has inside a list of elements and forall exprs.
	private ParseTreeNode predicates; //post-instantation side conditions.
	private boolean isGoalStep; //si es un paso especial para determinar si un item es meta o no.
	
	public DeductiveStepNode( String name , AntecedentNode antecedent , AntecedentNode negations , ParseTreeNode consequent , ParseTreeNode conditions , ParseTreeNode predicates , boolean isGoalStep )
	{
		this.name = name;
		this.antecedent = antecedent;
		this.negations = negations;
		this.consequent = consequent;
		this.conditions = conditions;
		this.predicates = predicates;
		this.isGoalStep = isGoalStep;
	}
	


	public String toString()
	{
		String result="";
		result += "Step: " + name;
		result += "\n\n";
		result += antecedent.toString();
		result += "\n------------------------";
		result += conditions.toString();
		result += " / ";
		result += predicates.toString();
		result += "\n";
		result += consequent.toString();
		return result;
	}
	
	public int numChildren()
	{
		return 5;
	}
	
	public ParseTreeNode getChild(int i)
	{
		switch(i)
		{
			case 0: return antecedent;
			case 1: return negations;
			case 2: return conditions;
			case 3: return predicates;
			case 4: return consequent;
			default: return null;
		}
	}
	
	public AntecedentNode getAntecedent()
	{
		return antecedent;
	}
	
	public ParseTreeNode getConsequent()
	{
		return consequent;
	}
	
	public ParseTreeNode getConditions()
	{
		return conditions;
	}
	
	public ParseTreeNode getPredicates()
	{
		return predicates;
	}
	
	public ParseTreeNode getNegations()
	{
		return negations;
	}
	
	public String getName()
	{
		return name;
	}
	
	/**
	 * Nos dice si ste es un paso deductivo normal (en cuyo caso devuelve false)
	 * o un paso de meta, que sirve para determinar si un item es meta del algoritmo
	 * o no (en cuyo caso devuelve true).
	 * @return
	 */
	public boolean isGoalStep()
	{
		return isGoalStep;
	}
	
}
