package parser.sparser.tree;

import grammar.ContextFreeGrammar;

/**
 * Nodo raz de un rbol de esquema, representando el esquema
 * de anlisis sintctico en su conjunto.
 */
public class SchemaNode
{
	
	private DeductiveStepNode[] steps;
	
	private Class grammarClass;
	private Class agendaClass;
	private String engineClassName;
	
	public SchemaNode ( DeductiveStepNode[] steps  )
	{
		this.steps = steps;
		this.grammarClass = ContextFreeGrammar.class; //this line is superfluous, I think
	}
	
	/**
	 * Fija el tipo de gramtica con el que trabaja este esquema de anlisis
	 * sintctico a una clase Java dada, que deber implementar el interfaz
	 * Grammar.
	 * @param cl Clase que representa a una gramtica, implementando el interfaz
	 * Grammar.
	 */
	public void setGrammarClass( Class cl )
	{
		this.grammarClass = cl;
	}
	
	/**
	 * Fija el tipo de agenda con el que trabaja este esquema de anlisis
	 * sintctico a una clase Java dada, que deber implementar el interfaz
	 * Agenda.
	 * @param cl Clase que representa a una agenda, implementando el interfaz
	 * Agenda.
	 */
	public void setAgendaClass( Class cl )
	{
		this.agendaClass = cl;
	}
	
	/**
	 * Fija el tipo de mquina deductiva con el que trabaja este esquema de anlisis
	 * sintctico a una clase Java dada, que deber implementar el interfaz
	 * DeductiveEngine.
	 * @param cl Nombre de que representa a una agenda, implementando el interfaz
	 * DeductiveEngine.
	 */
	public void setEngineClassName( String cl )
	{
		this.engineClassName = cl;
	}
	
	/**
	 * Devuelve el tipo de gramtica con el que trabaja esate esquema de anlisis
	 * sintctico.
	 * @return Clase que implementa el interfaz Grammar y que representa el tipo
	 * de gramtica con el que trabaja el esquema de anlisis.
	 */
	public Class getGrammarClass()
	{
		return grammarClass;
	}
	
	/**
	 * Devuelve el tipo de agenda con el que trabaja este esquema de anlisis
	 * sintctico.
	 * @return Clase que implementa el interfaz Agenda y que representa el tipo
	 * de agenda con el que trabaja el esquema de anlisis.
	 */
	public Class getAgendaClass()
	{
		return agendaClass;
	}
	
	/**
	 * Devuelve el tipo de mquina deductiva con el que trabaja este esquema de anlisis
	 * sintctico.
	 * @return Nombre de la clase que implementa el interfaz DeductiveEngine y que representa el tipo
	 * de mquina deductiva con el que trabaja el esquema de anlisis.
	 */
	public String getEngineClassName()
	{
		return engineClassName;
	}
	
	public String toString()
	{
		String result = "";
		for ( int i = 0 ; i < steps.length ; i++ )
		{
			if ( steps[i].isGoalStep() )
			{
				result += "Goal:";
				result += "\n";
			}
			result += steps[i].toString();
			result += "\n";
			if ( i < steps.length )
				result += "***************\n";		
		}
		return ( result );
	}
	
	public void printOut ( )
	{
		System.out.println(this.toString());
	}

	public int numChildren()
	{
		return steps.length;
	}
	
	public DeductiveStepNode getChild(int i)
	{
		return steps[i];
	}
	
}
